import requests
from eth_account import Account
from eth_account.messages import encode_defunct
from loguru import logger
from sybil_engine.contract.send import Send
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.utils.web3_utils import init_web3
from web3_wizzard_lib.core.utils.sub_module import SubModule


class OrbiterClaim(SubModule):
    module_name = 'ORBITER_CLAIM'

    nft_address = "0x13dFDd3a9B39323F228Daf73B62C23F7017E4679"

    def execute(self, account, chain='ARBITRUM'):

        # message to sign Orbiter Airdrop
        message_text = "Orbiter Airdrop"
        text_hex = "0x" + message_text.encode('utf-8').hex()
        text_encoded = encode_defunct(hexstr=text_hex)
        signed_message = Account.sign_message(text_encoded, private_key=account.key)

        signature = f'0x{signed_message.signature.hex()}'
        headers = {
            "token": signature
        }

        result = requests.post(
            "https://airdrop-api.orbiter.finance/airdrop/snapshot",
            headers=headers
        ).json()

        chain_id = result['result']['chainId']

        chain, data = self.form_data(chain, chain_id, result)

        try:
            self.send_transaction(account, chain, data)
        except Exception as e:
            self.send_transaction(account, chain, data.replace("0000000000000000014", "0000000000000000013"))

    def form_data(self, chain, chain_id, result):
        amount_str = result['result']['proof'][0]['amount']
        logger.info(f"Orbiter Airdrop snapshot result: {amount_str}")
        amount_int = int(amount_str.replace(".", ""))
        logger.info(f"AMOUNT INT: {amount_int}")
        hex_value = hex(amount_int)[2:]
        logger.info(f"AMOUNT: {hex_value}")
        logger.info(f"HEX {hex_value}")
        if int(chain_id) == 42161:
            chain = 'ARBITRUM'
            data = "0xfa5c4e99071cbb2ff029ddaf4b691745b2ba185cbe9ca2f5fa9e7358bada8fbdce764291"
            data += f"0000000000000000000000000000000000000000000000{hex_value}00000000000000000000000000000000000000000000000000000000000000600000000000000000000000000000000000000000000000000000000000000014"
        else:
            data = "0xfa5c4e995d32ba2a988b8fa6c2ae96d3b5980c67e8d8cfbf0d4c89479b79c1e277843438"
            data += f"000000000000000000000000000000000000000000000010b202b565eb37160000000000000000000000000000000000000000000000000000000000000000600000000000000000000000000000000000000000000000000000000000000010"
            chain = 'BASE'
        for line_data in result['result']['proof'][0]['data']:
            data += line_data[2:]
        logger.info(data)
        return chain, data

    def send_transaction(self, account, chain, data):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)
        Send(None, web3).send_to_wallet(
            account,
            self.nft_address,
            NativeBalance(0, chain, "ETH"),
            data
        )

    def log(self):
        return "ORBITER NFT"

def round_to_custom_precision(number, precision):
    # Find the rounding factor (10^precision)
    factor = 10 ** precision
    # Perform rounding
    return round(number / factor) * factor


def truncate_to_decimal_places(value, decimals):
    factor = 10 ** decimals
    return int(float(int(value * factor)) / factor)


import requests
from sybil_engine.contract.send import Send
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.nft.sign import sign_in_message
from web3_wizzard_lib.core.utils.sub_module import SubModule


class TrustaPoh(SubModule):
    module_name = 'TRUSTA_POH'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['TRUSTA_POH']

        token_auth = sign_in_message(account, web3)
        txn_calldata = get_attest_data_humanity(token_auth)

        score = txn_calldata['message']['score']
        if score != -1:
            raise Exception(f'Score кошелька равен {score}, аттестация не выполняется')

        send = Send(contract_address, web3)
        send.send_to_wallet(
            account,
            contract_address,
            NativeBalance(txn_calldata['calldata']['value'], chain, 'ETH'),
            txn_calldata['calldata']['data']
        )

    def log(self):
        return "TRUSTA POH (Group A)"


def get_attest_data_humanity(token_auth):
    url = 'https://mp.trustalabs.ai/accounts/attest_calldata?attest_type=humanity'
    headers = {'Authorization': f'TOKEN {token_auth}', 'Accept': 'application/json'}
    r = requests.get(url, headers=headers)
    if r.status_code == 200:
        res = [r.json()]
        if res[0]['code'] == 0:
            txn_calldata = res[0]['data']
            return txn_calldata




from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.utils.utils import ConfigurationException

from web3_wizzard_lib.core.contract.cog_erc20 import CogErc20Contract
from web3_wizzard_lib.core.modules.bank.bank import Bank


class Cog(Bank):
    app_name = 'COG_USDC'
    supported_chains = ['SCROLL']

    def __init__(self, chain, web3):
        self.contract_address = get_contracts_for_chain(chain)[self.app_name]
        self.contract = CogErc20Contract(self.contract_address, web3)

    def supply(self, account, amount):
        raise ConfigurationException("Only redeem supported for Compound V3")

    def redeem(self, account, amount, token):
        self.contract.redeem(account, amount)

    def get_deposit_amount(self, account, token):
        return self.contract.balance_of(account)



import random

from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance, set_rpc_for_chain
from sybil_engine.module.module import RepeatableModule, Order
from sybil_engine.utils.package_import_utils import import_all_modules_in_directory, get_all_subclasses
from sybil_engine.utils.scan_utils import find_interacted_contracts
from sybil_engine.utils.utils import ConfigurationException, randomized_sleeping, RetryException, print_exception_chain
from sybil_engine.utils.validation_utils import validate_interval

from web3_wizzard_lib.core.modules.nft.nft_submodule import NftSubmodule
from web3_wizzard_lib.core.utils.sub_module import SubModule


class NFTMinter(RepeatableModule):
    module_name = 'NFT_MINTER'
    module_config = 'nft_minter'
    random_order = Order.RANDOM
    repeat_conf = 'nft_amount_interval'
    sleep_after_var = True
    retries = 100

    @RepeatableModule.repeatable_log
    def execute(self, nft_module_names, nft_amount_interval, unique_nft, skip_delay, account):
        self.sleep_after_var = True
        self.storage.setdefault("minted_nfts", [])
        minted_nfts = self.storage.get("minted_nfts")

        nft_modules_without_minted = [item for item in set(nft_module_names) if item not in minted_nfts]
        nfts = [nft for nft in self.get_nfts() if nft and nft.module_name in nft_modules_without_minted]

        if nft_amount_interval['to'] > len(nft_module_names):
            raise ConfigurationException(
                f"Interval {nft_amount_interval} to > amount of NFT nfts allowed {len(nft_module_names)}")

        inited_module = random.choice(nfts)()

        if unique_nft and not hasattr(inited_module, "allow_reuse_address"):
            chain = 'LINEA'
            interactions = find_interacted_contracts(
                account.address,
                get_chain_instance(chain)['api_scan'],
                get_chain_instance(chain)['api_scan_key']
            )

            nft_contract_address = self.get_nft_address(chain, inited_module)

            if nft_contract_address in interactions:
                logger.info(f"{inited_module.log()} already minted for {account.address}")
                self.sleep_after_var = False
                randomized_sleeping(skip_delay)
                return

        logger.info(f"Mint {inited_module.log()}")
        try:
            inited_module.execute(account)
        except SkipRetryException as e:
            self.sleep_after_var = False
            logger.info(e)
            return
        minted_nfts.append(inited_module.module_name)
        self.storage.put("minted_nfts", minted_nfts)

    def get_nfts(self):
        import_all_modules_in_directory("utils.modules.nft")
        return get_all_subclasses(SubModule) + get_all_subclasses(NftSubmodule)

    def get_nft_address(self, chain, inited_module):
        if inited_module.module_name in get_contracts_for_chain(chain):
            return get_contracts_for_chain(chain)[inited_module.module_name]
        else:
            return inited_module.nft_address

    def log(self):
        return "NFT MINTER"

    def parse_params(self, module_params):
        validate_interval(module_params['nft_amount_interval'])

        if 'skip_delay' not in module_params:
            module_params['skip_delay'] = {'from': 10, 'to': 30}

        if 'unique_nft' not in module_params:
            module_params['unique_nft'] = True

        return module_params['nft_modules'], module_params['nft_amount_interval'], module_params['unique_nft'], \
            module_params['skip_delay']

    retry_counter = 0

    def handle(self, e):
        print_exception_chain(e)

        if self.retry_counter < self.retries:
            self.retry_counter = self.retry_counter + 1
            logger.info(f"{self.retry_counter} retries from {self.retries}")
            set_rpc_for_chain('LINEA')
            randomized_sleeping({'from': 1, 'to': 60})
            raise RetryException(e)

    def sleep_after(self):
        return self.sleep_after_var


class SkipRetryException(Exception):
    pass



from loguru import logger
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.balance_utils import interval_to_eth_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.retry import retry

from web3_wizzard_lib.core.contract.nile_contract import NileContract
from web3_wizzard_lib.core.modules.liquidity.pool import Pool


class NilePool(Pool):
    pool_contract = 'NILE_POOL'

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.nile_contract = NileContract(self.velocore_pool_contract_address, self.web3)

    @retry(max_attempts=2, retry_interval={'from': 10, 'to': 20})
    def deposit(self, amount_interval, account, token, min_native_balance, chain):
        amount = interval_to_eth_balance(amount_interval, account, chain, self.web3)

        pool_amount = self.nile_contract.get_amount_out(
            amount,
            get_tokens_for_chain(chain)['WETH'],
            get_tokens_for_chain(chain)[token]
        )[0]

        nile_token = Erc20Token(chain, token, self.web3)

        if nile_token.allowance(account, self.nile_contract.contract_address) < nile_token.balance(account).wei:
            nile_token.approve(account, self.nile_contract.contract_address)

        logger.info(f"Deposit {amount} and {nile_token.balance(account)}")
        self.nile_contract.add_liquidity_eth(account, nile_token.erc20_contract.contract_address, pool_amount, amount)

    @retry(max_attempts=2, retry_interval={'from': 10, 'to': 20})
    def withdraw(self, account, token, chain):
        weth_zero_address = get_tokens_for_chain(chain)[f"WETH_{token}_LP"]

        weth_zero = Erc20Token(chain, weth_zero_address, self.web3)

        if weth_zero.balance(account).wei < 10000:
            logger.info("NILE almost zero balance")

        zero_token_address = get_tokens_for_chain(chain)[token]

        amount = self.nile_contract.quote_remove_liquidity_eth(
            get_tokens_for_chain(chain)['WETH'],
            get_tokens_for_chain(chain)[token],
            weth_zero.balance(account).wei
        )

        if weth_zero.allowance(account, self.nile_contract.contract_address) < weth_zero.balance(account).wei:
            weth_zero.approve(account, self.nile_contract.contract_address)

        logger.info(f"Withdraw {token} on NILE {weth_zero.balance(account)}")

        self.nile_contract.remove_liquidity_eth(
            account,
            zero_token_address,
            weth_zero.balance(account).wei,
            amount[0],
            amount[1]
        )



import random
import string

import requests
from loguru import logger
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.utils.app_account_utils import AppAccount

from web3_wizzard_lib.core.contract.scroll_canvas_mint_contract import ScrollCanvasMintContract
from web3_wizzard_lib.core.modules.nft.nft_submodule import NftSubmodule


class ScrollCanvasMint(NftSubmodule):
    module_name = 'SCROLL_CANVAS_MINT'
    nft_address = '0xB23AF8707c442f59BDfC368612Bd8DbCca8a7a5a'

    def execute(self, account: AppAccount, wei_cost=from_eth_to_wei(0.0005), chain='SCROLL'):
        contract = ScrollCanvasMintContract(
            '0xB23AF8707c442f59BDfC368612Bd8DbCca8a7a5a',
            self.create_web3(account, chain)
        )

        username = self.generate_username(random.randint(8, 12))
        referral = random.choice(self.get_referrals())

        logger.info(f"Mint Scroll Canvas with username {username} using ref {referral}")

        contract.mint(
            account,
            username,
            requests.get(f'https://canvas.scroll.cat/code/{referral}/sig/{account.address}').json()['signature'],
            wei_cost
        )

    def log(self):
        return "SCROLL CANVAS MINT"

    def generate_username(self, length):
        # Combine uppercase, lowercase letters and digits
        characters = string.ascii_letters + string.digits
        # Generate random string
        random_string = ''.join(random.choice(characters) for _ in range(length))
        return random_string

    def get_referrals(self):
        return [
            'POPNK',
            'GTNPB',
            'KZFXI',
            'MFWQ7'
        ]



from json import dumps

import requests
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.nidum_contract import NidumContract
from web3_wizzard_lib.core.modules.nft.sign import sign_msg
from web3_wizzard_lib.core.utils.sub_module import SubModule


class Nidum(SubModule):
    module_name = 'NIDUM'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['NIDUM']
        nidus_nft = NidumContract(contract_address, web3)

        claim_data = request_ops(account, web3)

        nidus_nft.mint_nft(account, claim_data)

    def log(self):
        return "NIDUM NFT"


def register_wallet(wallet):
    url = f'https://auth.sidusheroes.com/api/v1/users'
    headers = {'Content-Type': 'application/json'}
    data = {'address': wallet.address.lower()}
    json_data = dumps(data)
    requests.post(url, data=json_data, headers=headers)


def get_msg(wallet):
    url = 'https://auth.sidusheroes.com/api/v1/users/' + wallet.address.lower()
    r = requests.get(url)
    response = r.json()
    nonce = response['data']['nonce']
    msg_text = f'Please sign this message to connect to sidusheroes.com: {nonce}'
    return msg_text


def auth(wallet, signature):
    data = {"address": f"{wallet.address}", "signature": f"{signature}"}
    json_data = dumps(data)
    url = 'https://auth.sidusheroes.com/api/v1/auth'
    headers = {'Content-Type': 'application/json'}
    r = requests.post(url, data=json_data, headers=headers)
    response = r.json()
    bearer = response['data']['accessToken']
    return bearer


def get_token_data(wallet, bearer):
    url = f'https://plsrv.sidusheroes.com/shadow-game-linea/api/v1/item'
    headers = {'Content-Type': 'application/json', 'Authorization': f'Bearer {bearer}'}
    data = {"user": f"{wallet.address.lower()}", "contract": "0x34Be5b8C30eE4fDe069DC878989686aBE9884470", "tokenId": 9}
    json_data = dumps(data)
    r = requests.post(url, data=json_data, headers=headers)


def get_claim_data(wallet, bearer):
    url = 'https://plsrv.sidusheroes.com/shadow-game-linea/api/v1/claim'
    headers = {'Content-Type': 'application/json', 'Authorization': f'Bearer {bearer}',
               'Content-Length': '151', 'If-None-Match': 'W/"81-IPXBWNB48bs1CNK6NL+XgeHJooA"'}
    data = {"contract": "0x34Be5b8C30eE4fDe069DC878989686aBE9884470",
            "user": f'{wallet.address.lower()}',
            "tokensData": [{"tokenId": 9, "amount": 1}]}
    json_data = dumps(data)
    r = requests.post(url, data=json_data, headers=headers)
    response = r.json()
    return response


def request_ops(wallet, web3):
    register_wallet(wallet)
    msg_text = get_msg(wallet)
    signature = sign_msg(wallet, msg_text, web3)
    bearer = auth(wallet, signature)
    get_token_data(wallet, bearer)
    claim_data = get_claim_data(wallet, bearer)
    return claim_data



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/bungee.json")

bungee_chain_ids = {
    "BSC": 56,
    "OPTIMISM": 10,
    "POLYGON": 137,
    "BASE": 8453,
    "ARBITRUM": 42161,
    "AVALANCHE": 43114,
    "ZK_EVM": 1101,
    "ZKSYNC": 324
}


class Bungee(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def refuel(self, account, to_chain, amount):
        sender = account.address

        txn_params = self.build_generic_data(sender, False)
        txn_params['value'] = amount

        return self.contract.functions.depositNativeToken(
            bungee_chain_ids[to_chain],
            sender
        ).build_transaction(txn_params)



import random
import string

from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.kreatorland_contract import KreatorLandContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class KreatorLand(SubModule):
    module_name = 'KREATOR_LAND'

    def execute(self, account, chain='ZKSYNC'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['KREATOR_LAND']
        kreator_land_contract = KreatorLandContract(contract_address, web3)

        uri = f'https://cloudflare-ipfs.com/ipfs/{random_string}/metadata.json'

        kreator_land_contract.mint(account, uri)

    def log(self):
        return "Kreator Land"

    def parse_params(self, module_params):
        return []


def random_string(length=59):
    characters = string.ascii_letters + string.digits  # A-Z, a-z, 0-9
    return ''.join(random.choice(characters) for _ in range(length))



from loguru import logger
from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/stargate_token_pool.json")

MAX_ALLOWANCE = 115792089237316195423570985008687907853269984665640564039457584007913129639935


class StargateTokenPool(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def approve_pool(self, account, contract_on_approve):
        logger.info(f"Approving token for pool")

        sender = account.address

        txn_params = self.build_generic_data(sender, set_contract_address=False)

        contract_txn = self.contract.functions.approve(
            Web3.to_checksum_address(contract_on_approve),
            MAX_ALLOWANCE
        ).build_transaction(txn_params)

        contract_txn['gas'] = int(self.web3.eth.estimate_gas(contract_txn) * 1.1)

        return contract_txn

    def balance_of(self, account):
        return self.contract.functions.balanceOf(account.address).call()

    def approve(self, account, contract_on_approve):
        logger.info(f"Approving token")

        txn_params = self.build_generic_data(account.address, False)

        return self.contract.functions.approve(
            Web3.to_checksum_address(contract_on_approve),
            MAX_ALLOWANCE
        ).build_transaction(txn_params)

    def allowance(self, account, allowance_contract):
        return self.contract.functions.allowance(account.address, Web3.to_checksum_address(allowance_contract)).call()



from random import choices
from string import ascii_letters, digits

import requests
from eth_utils import keccak
from faker import Faker
from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.retry import retry
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.linea_ens_names_contract import LineaEnsNamesContract
from web3_wizzard_lib.core.utils.sub_module import SubModule
from web3_wizzard_lib.core.modules.nft_minter import SkipRetryException


class LineaENSNames(SubModule):
    module_name = 'LINEA_ENS'
    nft_address = '0xDb75Db974B1F2bD3b5916d503036208064D18295'

    def execute(self, account: AppAccount, chain='LINEA'):
        self.sleep_after_conf = True

        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract = LineaEnsNamesContract(self.nft_address, web3)

        linea_name = self.get_available_name(contract)

        if contract.redeemed(account):
            #logger.info("[+] Web3 | Wallet already redeemed Linea name.")
            raise SkipRetryException("[+] Web3 | Wallet already redeemed Linea name.")

        logger.info(f"Minting {linea_name}.linea.eth for {account.address}")
        duration = 60 * 60 * 24 * 365 * 3  # 3 years
        secret, data = self.commit(account, linea_name, duration, contract)

        # Uncomment the following line to perform the registration
        self.register_poh(account, contract, data, duration, linea_name, secret)

    def get_available_name(self, contract):
        while True:
            linea_name = self.generate_linea_name()
            if contract.available(linea_name):
                break
            logger.warning(f'[-] Web3 | Linea name "{linea_name}.linea.eth" is unavailable. Retrying...')
        return linea_name

    @retry(max_attempts=20, retry_interval={'from': 10, 'to': 20})
    def register_poh(self, account, contract, data, duration, linea_name, secret):
        contract.register_poh(account, linea_name, duration, secret, data, self.get_poh_signature(account))

    def generate_linea_name(self):
        return Faker().user_name().replace('_', '-')

    def log(self):
        return "LINEA ENS"

    def commit(self, account, linea_name: str, duration: int, linea_ens_contract):
        secret = "0x" + ''.join(choices(ascii_letters + digits, k=10)).encode().hex().ljust(64, '0')
        hashed_name = self.namehash(f"{linea_name}.linea.eth")
        data = linea_ens_contract.addr_contract.functions.setAddr(
            hashed_name, 60,
            account.address
        )._encode_transaction_data()

        commitment = linea_ens_contract.make_commitment(
            account,
            linea_name,
            duration,
            secret,
            data,
        )

        linea_ens_contract.commit(account, commitment)

        return secret, data

    def namehash(self, name: str):
        labels = name.split('.')
        labels.reverse()
        result = "0000000000000000000000000000000000000000000000000000000000000000"

        for label in labels:
            hashed = keccak(text=label).hex()
            result = keccak(hexstr=result + hashed).hex()

        return "0x" + result

    def sleep_after_conf(self):
        return self.sleep_after_conf

    def get_poh_signature(self, account):
        headers = {
            "Origin": "https://relay.link",
            "Referer": "https://relay.link/"
        }

        r = requests.get(f'https://linea-poh-signer-api.linea.build/poh/{account.address}', headers=headers)
        if len(r.text) == 132 and r.text.startswith("0x"):
            return {"sign": r.text, "success": True, "msg": ""}
        elif "address not POH" in r.text:
            return {"sign": "", "success": False, "msg": "This wallet dont have POH"}
        else:
            raise Exception(r.text)



from sybil_engine.config.app_config import get_cex_data, get_cex_conf
from sybil_engine.domain.cex.binance import Binance
from sybil_engine.domain.cex.okx import OKX
from sybil_engine.module.module import Module
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.utils import ConfigurationException


class CEXSubAccountTransfer(Module):
    module_name = 'CEX_SUB_ACCOUNT_TRANSFER'
    module_config = 'cex_account_transfer_config'

    def execute(self, cex, account: AppAccount):
        password, cex_data = get_cex_data()

        tokens = ['ETH', 'CORE', 'POL', 'USDC', 'USDT', 'OKB', 'ZRO']

        if cex == 'okx':
            cex_obj = OKX(cex_data[get_cex_conf()], password)
        elif cex == 'binance':
            cex_obj = Binance(cex_data[get_cex_conf()], password)
        else:
            raise ConfigurationException(f"{cex} is not")

        cex_obj.transfer_from_sub_account(tokens)

    def log(self):
        return "CEX SUB ACCOUNT TRANSFER"

    def parse_params(self, module_params):
        if 'cex' not in module_params:
            module_params['cex'] = 'okx'

        return [module_params['cex']]



import csv
import datetime
import os
from googleapiclient.discovery import build
from loguru import logger
from sybil_engine.utils.google_utils import get_google_credentials

from sybil_engine.utils.config_utils import get_config

statistic_date_string = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")


class StatisticsWriter:
    def init_if_required(self, job_name, header):
        pass

    def write_row(self, job_name, row):
        pass


class CsvStatisticsWriter(StatisticsWriter):
    def init_if_required(self, job_name, header):
        output_file = self._get_file_name(job_name)
        file_exists = os.path.isfile(output_file)

        rows_to_write = []

        if not file_exists or os.path.getsize(output_file) == 0:
            rows_to_write.append(header)

        with open(output_file, mode='a', newline='') as file:
            writer = csv.writer(file)
            writer.writerows(rows_to_write)

    def write_row(self, job_name, row):
        rows_to_write = []
        output_file = self._get_file_name(job_name)

        rows_to_write.append(row)

        with open(output_file, mode='a', newline='') as file:
            writer = csv.writer(file)
            writer.writerows(rows_to_write)

    def _get_file_name(self, job_name):
        return f'data/csv/{job_name}.csv'


class GoogleStatisticsWriter(StatisticsWriter):
    def __init__(self, sheet):
        self.SCOPES = ['https://www.googleapis.com/auth/spreadsheets']
        self.credentials = get_google_credentials()
        self.service = build('sheets', 'v4', credentials=self.credentials)

        self.sheet = sheet

    def copy_sheet(self, source_sheet_name, destination_sheet_name):
        # Retrieve the spreadsheet data to locate the source sheet
        spreadsheet = self.service.spreadsheets().get(spreadsheetId=self.sheet).execute()
        source_sheet_info = next(
            (sheet for sheet in spreadsheet['sheets'] if sheet['properties']['title'] == source_sheet_name),
            None
        )
        if source_sheet_info is None:
            raise ValueError(f"Sheet '{source_sheet_name}' not found.")
        source_sheet_id = source_sheet_info['properties']['sheetId']

        # Copy the sheet to the same spreadsheet (appended as the rightmost tab)
        request_body = {
            'destinationSpreadsheetId': self.sheet
        }
        result = self.service.spreadsheets().sheets().copyTo(
            spreadsheetId=self.sheet,
            sheetId=source_sheet_id,
            body=request_body
        ).execute()
        new_sheet_id = result['sheetId']

        # Update the new sheet's properties: rename it and set its index to 0 (leftmost)
        body = {
            'requests': [
                {
                    'updateSheetProperties': {
                        'properties': {
                            'sheetId': new_sheet_id,
                            'title': destination_sheet_name,
                            'index': 0
                        },
                        'fields': 'title,index'
                    }
                }
            ]
        }
        self.service.spreadsheets().batchUpdate(spreadsheetId=self.sheet, body=body).execute()

        logger.info(f"Sheet copied. New sheet '{destination_sheet_name}' placed at the leftmost position.")

    def init_if_required(self, sheet_name, header):
        if not self._sheet_exists(sheet_name):
            self._create_new_sheet(sheet_name)
            self.write_row(sheet_name, header)

    def _sheet_exists(self, sheet_name):
        try:
            self.service.spreadsheets().values().get(spreadsheetId=self.sheet, range=sheet_name).execute()
            logger.info('Sheet exists.')
            return True
        except Exception:
            return False

    def _create_new_sheet(self, sheet_name):
        body = {'requests': [{'addSheet': {'properties': {'title': sheet_name}}}]}
        self.service.spreadsheets().batchUpdate(spreadsheetId=self.sheet, body=body).execute()
        logger.info(f'Sheet "{sheet_name}" created.')

    def write_cell(self, sheet_name, cell_reference, value):
        range_ = f"{sheet_name}!{cell_reference}"
        body = {'values': [[value]]}
        self.service.spreadsheets().values().update(
            spreadsheetId=self.sheet, range=range_, valueInputOption='USER_ENTERED', body=body
        ).execute()
        logger.info(f"Value written to cell {cell_reference} in sheet {sheet_name}: {value}")

    def write_row(self, job_name, row):
        range_ = f"{job_name}!A1"
        rows_read = self.service.spreadsheets().values().get(spreadsheetId=self.sheet, range=range_).execute().get(
            'values', [])
        body = {'values': [row]}
        range_to_write = f"{job_name}!A{len(rows_read) + 1}"
        self.service.spreadsheets().values().append(
            spreadsheetId=self.sheet, range=range_to_write, valueInputOption='USER_ENTERED', body=body
        ).execute()
        logger.info(f'Row written: {row}')

    def get_leftmost_sheet(self):
        """
        Returns the name of the leftmost sheet in the spreadsheet.
        The leftmost sheet is the first sheet in the sheets collection.
        """
        spreadsheet = self.service.spreadsheets().get(spreadsheetId=self.sheet).execute()
        if 'sheets' in spreadsheet and len(spreadsheet['sheets']) > 0:
            return spreadsheet['sheets'][0]['properties']['title']
        return None


def get_statistic_writer():
    if get_config("STATISTICS_MODE") == "CSV":
        return CsvStatisticsWriter()
    else:
        return GoogleStatisticsWriter(get_config("STATS_SPREADSHEET_ID"))



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/velocore_lens.json")


def to_bytes32(address):
    return Web3.to_bytes(hexstr=address).rjust(32, b'\0')


class VelocoreLensContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def getPoolBalance(self):
        token_identifier = '0xe2c67A9B15e9E7FF8A9Cb0dFb8feE5609923E5DB'

        # 0x000000000000000000000000e2c67A9B15e9E7FF8A9Cb0dFb8feE5609923E5DB

        return (self.contract.functions.getPoolBalance(
            '0xe2c67A9B15e9E7FF8A9Cb0dFb8feE5609923E5DB',
            Web3.to_bytes(hexstr='0x000000000000000000000000b98308D11E2B578858Fbe65b793e71C7a0CAa43e')
        )
                .call())

    def userBalances(self):
        token_identifier = '0xe2c67A9B15e9E7FF8A9Cb0dFb8feE5609923E5DB'

        # 0x000000000000000000000000e2c67A9B15e9E7FF8A9Cb0dFb8feE5609923E5DB
        # 0x000000000000000000000000e2c67A9B15e9E7FF8A9Cb0dFb8feE5609923E5DB

        return (self.contract.functions.userBalances(
            '0xe2c67A9B15e9E7FF8A9Cb0dFb8feE5609923E5DB',
            [Web3.to_bytes(hexstr='0x000000000000000000000000b98308D11E2B578858Fbe65b793e71C7a0CAa43e')]
        )
                .call())

    def query_gauge(self, pool_address, account):
        return (self.contract.functions.queryGauge(
            pool_address,
            account.address
        )
                .call())



import random

from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.readon_contract import ReadOnContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class ReadOn(SubModule):
    module_name = 'READON'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['READON']
        readon_curate = ReadOnContract(contract_address, web3)

        low = 1700890697805328281
        high = 1709890697805328281

        # Generate a random number within the given range
        contentUrl = format(random.randint(low, high), 'x').zfill(64)

        readon_curate.curate(account, contentUrl)

    def log(self):
        return "READON NFT"



import requests
from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from web3 import Web3


class XYSwapContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, None)

    def get_quote(self, from_token: str, to_token: str, amount: int, slippage: float):
        url = "https://aggregator-api.xy.finance/v1/quote"

        params = {
            "srcChainId": self.web3.eth.chain_id,
            "srcQuoteTokenAddress": Web3.to_checksum_address(from_token),
            "srcQuoteTokenAmount": amount.wei,
            "dstChainId": self.web3.eth.chain_id,
            "dstQuoteTokenAddress": Web3.to_checksum_address(to_token),
            "slippage": slippage
        }

        response = requests.get(url=url, params=params)
        transaction_data = response.json()

        return transaction_data

    @evm_transaction
    def swap(
            self,
            account,
            amount_to_swap,
            from_token: str,
            to_token: str,
            quote: int,
            slippage: int
    ):
        amount_wei = amount_to_swap.wei

        swap_provider = quote["routes"][0]["srcSwapDescription"]["provider"]

        transaction_data = self.build_transaction(
            account,
            from_token,
            to_token,
            amount_wei,
            slippage,
            swap_provider
        )

        tx_data = self.build_generic_data(account.address, True)

        tx_data.update(
            {
                "to": self.web3.to_checksum_address(transaction_data["tx"]["to"]),
                "data": transaction_data["tx"]["data"],
                "value": transaction_data["tx"]["value"],
                "nonce": self.web3.eth.get_transaction_count(account.address)
            }
        )

        return tx_data

    def build_transaction(self, account, from_token: str, to_token: str, amount: int, slippage: float,
                          swap_provider: str):
        url = "https://aggregator-api.xy.finance/v1/buildTx"

        params = {
            "srcChainId": self.web3.eth.chain_id,
            "srcQuoteTokenAddress": Web3.to_checksum_address(from_token),
            "srcQuoteTokenAmount": amount,
            "dstChainId": self.web3.eth.chain_id,
            "dstQuoteTokenAddress": Web3.to_checksum_address(to_token),
            "slippage": slippage,
            "receiver": account.address,
            "srcSwapProvider": swap_provider,
        }

        # if XYSWAP_CONTRACT["use_ref"]:
        #     params.update({
        #         "affiliate": Web3.to_checksum_address("0xb98308D11E2B578858Fbe65b793e71C7a0CAa43e"),
        #         "commissionRate": 10000
        #     })

        response = requests.get(url=url, params=params)

        return response.json()



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/compound_v3.json")


class CompoundV3Contract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def borrow_balance_of(self, account):
        return self.contract.functions.borrowBalanceOf(account.address).call()

    @evm_transaction
    def supply(self, account, amount, token_address):
        txn_params = self.build_generic_data(account.address, set_contract_address=False)

        return self.contract.functions.supply(
            token_address,
            amount
        ).build_transaction(txn_params)

    def user_collateral(self, account, token_address):
        return self.contract.functions.userCollateral(
            account.address,
            token_address
        ).call()[0]



import random

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.data.pairs import Pairs
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.module.module import Module
from sybil_engine.utils.utils import randomized_sleeping
from sybil_engine.utils.validation_utils import validate_interval, validate_token
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.swap.swap_list import swap_facade


class SellAll(Module):
    module_name = 'SELL_ALL'
    module_config = 'sell_all_config'
    allowed_chains = ['ZKSYNC', 'LINEA', 'BASE', 'SCROLL', 'ARBITRUM']

    def execute(self, chain, sleep_interval, receive_token, account):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        pairs = Pairs(swap_facade)

        for pair, swaps in pairs.get_all_pair_swaps(chain, pairs.get_pair_names(chain, receive_token), receive_token):
            erc20_token = Erc20Token(chain, pair['tokens'][1], web3)

            amount_to_sell = erc20_token.balance(account)

            random_app = random.choice(swaps)

            logger.info(f"Sell all on {random_app}")

            if amount_to_sell.wei > 100:
                swap_facade.swap(account, amount_to_sell, chain_instance, pair, random_app, amount_to_sell.token,
                                 pair['tokens'][0], web3)
                randomized_sleeping(sleep_interval)

    def log(self):
        return "SELL ALL TOKENS"

    def parse_params(self, module_params):
        self.validate_supported_chain(module_params['chain'])
        validate_interval(module_params['sleep_interval'])
        validate_token(module_params['receive_token'])

        return module_params['chain'], module_params['sleep_interval'], module_params['receive_token']

    def sleep_after(self):
        return False



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/pancake/factory.json")


class PancakePool(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def get_pool(self, from_token: str, to_token: str):

        pool = self.contract.functions.getPool(
            from_token,
            to_token,
            500
        ).call()

        return pool



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/rubyscore.json")


class RubyscoreContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def attest_rubyscore(self, account, signature, schemaId, expirationDate):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = 500000000000000

        signature_bytes = Web3.to_bytes(hexstr=signature[2:])

        contract_txn = self.contract.functions.attestRubyscore(
            (
                Web3.to_bytes(hexstr=schemaId[2:]),
                expirationDate,
                Web3.to_bytes(hexstr=f'000000000000000000000000{account.address[2:]}'),
                Web3.to_bytes(hexstr='0000000000000000000000000000000000000000000000000000000000000000'),
            ),
            [signature_bytes]  # Encapsulate in a list to match bytes[]
        ).build_transaction(txn_params)

        return contract_txn



import requests
from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.retry import retry
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.nomis_attest_contract import NomisAttestContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class NomisAttest(SubModule):
    module_name = 'NOMIS_ATTEST'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['NOMIS_ATTEST']

        url_template = f'http://nomis.cc/api/proxy/verax/attestation-data?address={account.address}&nonce=0'

        response = self.get_attest(url_template)

        mintedScore = response['data']['value']

        if mintedScore < 3000:
            formatted_score = "{:.2f}".format(mintedScore / 100)
            logger.info(f"Minted score is  {formatted_score}, minimum 30 is required, skip account")
            return

        schema = response['data']['schema']
        expirationTime = response['data']['expirationTime']
        revocable = response['data']['revocable']
        tokenId = response['data']['tokenId']
        updated = response['data']['updated']
        value = response['data']['value']
        chainId = response['data']['chainId']
        calcModel = response['data']['calcModel']
        validationPayload = response['data']['validationPayload']

        nomis_attest_contract = NomisAttestContract(contract_address, web3)

        nomis_attest_contract.attest(
            account,
            schema,
            expirationTime,
            revocable,
            tokenId,
            updated,
            value,
            chainId,
            calcModel,
            validationPayload
        )

    @retry(max_attempts=15, retry_interval={'from': 60 * 1, 'to': 60 * 4})
    def get_attest(self, url_template):
        response = requests.get(url_template)
        if response.status_code == 200:
            return response.json()
        else:
            raise Exception(f"HTTP error {response.json()}")

    def log(self):
        return "ATTEST NOMIS SCORE"



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/tavaera_id.json")


class TavaeraID(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def mint_citizen_id(self, account):
        txn_params = self.build_generic_data(account.address, False)
        txn_params['value'] = Web3.to_wei(0.0003, "ether")

        contract_txn = self.contract.functions.mintCitizenId().build_transaction(txn_params)

        return contract_txn



from venv import logger

from sybil_engine.data.networks import get_chain_instance
from sybil_engine.module.module import Module
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.validation_utils import validate_amount_interval
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.liquidity.syncswap_pool import SyncswapPool
from web3_wizzard_lib.core.modules.liquidity.velocore_pool import VelocorePool
from web3_wizzard_lib.core.modules.liquidity.nile_pool import NilePool
from web3_wizzard_lib.core.modules.liquidity.zerolend_liquidity import ZeroLendLiquidity


class LiquidityPool(Module):
    module_name = 'LIQUIDITY_POOL'
    module_config = 'liquidity_pool'

    def execute(self, action, amount_interval, token, dex, chain, account: AppAccount):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        if dex == 'syncswap':
            pool = SyncswapPool(chain_instance, web3)
        elif dex == 'velocore':
            pool = VelocorePool(chain_instance, web3)
        elif dex == 'nile':
            pool = NilePool(chain_instance, web3)
        elif dex == 'zerolend':
            pool = ZeroLendLiquidity(chain_instance, web3)
        else:
            raise Exception(f" {dex} Not supported")

        if action == 'DEPOSIT':
            logger.info(f"Deposit {token} to {dex}")
            pool.deposit(amount_interval, account, token, self.min_native_balance, chain)
        elif action == 'WITHDRAW':
            pool.withdraw(account, token, chain)
        else:
            raise Exception(f"{action} action not supported")

    def log(self):
        return "LIQUIDITY_POOL"

    def parse_params(self, module_params):
        validate_amount_interval(module_params['amount'])

        return [
            module_params['action'],
            module_params['amount'],
            module_params['token'],
            module_params['dex'],
            module_params['chain']
        ]


class EmptyDepositException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)



from sybil_engine.module.module import Module, Order

from web3_wizzard_lib.core.modules.stargate.bridge import StargateBridge


class StargateBridgeModule(Module):
    module_name = 'STARGATE_CLASSIC_BRIDGE'
    module_config = 'stargate_classic_config'
    random_order = Order.STRICT

    def execute(self, bridge_amount_interval, bridge_token, retry_interval, from_chain, to_chain, account):
        StargateBridge(retry_interval).bridge(
            self.min_native_balance,
            account,
            bridge_amount_interval,
            bridge_token,
            from_chain,
            to_chain
        )

    def log(self):
        return "STARGATE CLASSIC"

    def parse_params(self, module_params):
        if 'retry_interval' not in module_params:
            module_params['retry_interval'] = {'from': 60 * 5, 'to': 60 * 10}

        return (
            module_params['bridge_amount_interval'],
            module_params['bridge_token'],
            module_params['retry_interval'],
            module_params['from_chain'],
            module_params['to_chain']
        )



from sybil_engine.module.module import Order, Module

from web3_wizzard_lib.core.modules.bridge.coredao_bridge_sub import CoreDaoBridgeSubModule


class CoreDaoBridge(Module):
    module_name = 'COREDAO_BRIDGE'
    allowed_chains = ['POLYGON', 'COREDAO']
    module_config = 'coredao_bridge_config'

    def execute(self, bridge_amount_interval, from_chain, to_chain, token, account):
        CoreDaoBridgeSubModule().execute(
            bridge_amount_interval,
            from_chain,
            to_chain,
            token,
            self.min_native_balance,
            account
        )

    def log(self):
        return "COREDAO BRIDGE"

    def parse_params(self, module_params):
        self.validate_supported_chain(module_params['from_chain'])
        self.validate_supported_chain(module_params['to_chain'])

        return module_params['bridge_amount_interval'], module_params['from_chain'], module_params['to_chain'], \
            module_params['token']

    def order(self):
        return Order.RANDOM



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.tokens import Erc20Token

from web3_wizzard_lib.core.contract.layerbank_price import LayerBankPrice
from web3_wizzard_lib.core.contract.layerbank_token import LayerBankToken
from web3_wizzard_lib.core.contract.layerbankcontract import LayerBankContract
from web3_wizzard_lib.core.modules.bank.bank import Bank


class LayerBank(Bank):
    app_name = 'LAYERBANK'
    supported_chains = ['LINEA']

    def __init__(self, chain, web3):
        self.chain = chain
        self.web3 = web3
        self.layerbank = LayerBankContract(get_contracts_for_chain(chain)["LAYERBANK"], web3)

        lbwstETH = get_tokens_for_chain(chain)['lbwstETH']
        self.lbWstETHToken = LayerBankToken(lbwstETH, web3)

        lbETH = get_tokens_for_chain(chain)['lbETH']
        self.lbETHToken = LayerBankToken(lbETH, web3)

    def supply(self, account, token):
        token = LayerBankToken(get_tokens_for_chain(self.chain)[token], self.web3)

        erc20_token = Erc20Token(self.chain, token, self.web3)
        amount = erc20_token.balance(account)

        if 25 * token.one_decimal() <= self.layerbank.account_liquidity_of(account)[0]:
            logger.info("Already supplied over 25$ worth, skip this step")
            return

        if erc20_token.allowance(account, token.contract_address) < amount.wei:
            erc20_token.approve(account, token.contract_address)

        if amount.wei > 0:
            if token.contract_address not in self.layerbank.market_list_of(account):
                logger.info("Add support wstETH to LayerBank")
                self.layerbank.enter_markets(account, token.contract_address)

            logger.info(f"Supply {amount.log_line()}")

            self.layerbank.supply(account, token.contract_address, amount.wei)

        if 25 * token.one_decimal() > self.layerbank.account_liquidity_of(account)[0]:
            raise Exception(
                f"Not enough supply for 25$ borrow {self.layerbank.account_liquidity_of(account)[0] / token.one_decimal()}"
            )

    def borrow(self, account, token):
        token = LayerBankToken(get_tokens_for_chain(self.chain)[token], self.web3)

        layerbank_price = LayerBankPrice(get_contracts_for_chain(self.chain)["LAYERBANK_PRICE"], self.web3)

        usd_collateral = self.layerbank.account_liquidity_of(account)[0]
        eth_amount = (usd_collateral / layerbank_price.get_underlying_price(token.contract_address))

        logger.info(f"Borrow {eth_amount} {token.symbol()}")

        borrow_amount = int(
            eth_amount * token.one_decimal() * token.one_decimal() / token.exchange_rate())

        self.layerbank.borrow(
            account,
            token.contract_address,
            borrow_amount
        )

    def repay_borrow(self, account, token):
        token = LayerBankToken(get_tokens_for_chain(self.chain)[token], self.web3)

        layerbank_price = LayerBankPrice(get_contracts_for_chain(self.chain)["LAYERBANK_PRICE"], self.web3)

        borrowed_usdc = self.layerbank.account_liquidity_of(account)[2]
        lbETHPrice = layerbank_price.get_underlying_price(token.contract_address)
        ethAmount = int(
            (
                    borrowed_usdc / lbETHPrice) * token.one_decimal() * token.one_decimal() / token.exchange_rate()
        )

        if 2 * token.one_decimal() > borrowed_usdc:
            logger.info("Less than 2$ borrowed, skip")
            return

        logger.info(f"Repay Borrow {ethAmount / token.one_decimal()}")
        self.layerbank.repay_borrow(account, token.contract_address, ethAmount)

    def redeem(self, account, amount, token):
        if token == 'ETH':
            token = self.lbETHToken
        elif token == 'wstETH':
            token = self.lbWstETHToken

        underlying_balance = self.get_deposit_amount(account, token)

        if underlying_balance < 10000:
            logger.info("Less than 0.001$ supplied, skip")
            return

        self.layerbank.redeem_token(
            account,
            token.contract_address,
            int(underlying_balance * token.one_decimal() * 0.999 / token.exchange_rate())
        )

    def get_deposit_amount(self, account, token):
        if token == 'ETH':
            token = self.lbETHToken
        elif token == 'wstETH':
            token = self.lbWstETHToken

        return token.underlying_balance_of(account)



from datetime import datetime

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.data.networks import get_ids_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/velocore.json")


class VelocoreRouter(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def swap_exact_eth_for_tokens(self, account, amount_to_swap, token_in_address, token_out_address, amount_out_min):
        sender = account.address

        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)
        stable = False

        txn_params = self.build_generic_data(sender)

        txn_params['value'] = amount_to_swap.wei
        txn_params['data'] = self.contract.encode_abi('swapExactETHForTokens', args=(
            amount_out_min,
            [[token_in_address, token_out_address, stable]],
            sender,
            deadline
        )
                                                     )

        return txn_params

    @evm_transaction
    def swap_exact_tokens_for_eth(self, account, amount_to_swap, token_in_address, token_out_address, amount_out_min):
        sender = account.address
        txn_params = self.build_generic_data(account.address, False)

        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)
        stable = False

        return self.contract.functions.swapExactTokensForETH(
            amount_to_swap.wei,
            amount_out_min,
            [[token_in_address, token_out_address, stable]],
            sender,
            deadline
        ).build_transaction(txn_params)

    @evm_transaction
    def swap_exact_tokens_for_tokens(self, account, amount_to_swap, token_in_address, token_out_address,
                                     amount_out_min):
        sender = account.address
        txn_params = self.build_generic_data(sender)

        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)

        usdc_address = get_tokens_for_chain(get_ids_chain()[self.web3.eth.chain_id])['USDC']

        if token_out_address != usdc_address:
            stable = False
        else:
            stable = True

        return self.contract.functions.swapExactTokensForTokens(
            amount_to_swap.wei,
            amount_out_min,
            [[token_in_address, token_out_address, stable]],
            sender,
            deadline
        ).build_transaction(txn_params)

    def get_amount_out(self, amount_to_swap, token_in_address, token_out_address):
        return self.contract.functions.getAmountOut(
            amount_to_swap.wei,
            token_in_address,
            token_out_address
        ).call()[0]



from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.domain.balance.balance_utils import interval_to_eth_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.retry import retry

from web3_wizzard_lib.core.contract.zerolend_liq_contract import ZeroLendLiqContract
from web3_wizzard_lib.core.modules.liquidity.pool import Pool


class ZeroLendLiquidity(Pool):
    module_name = 'ZEROLEND_LIQUIDITY'
    pool_contract = 'ZEROLEND'

    @retry(max_attempts=5, retry_interval={'from': 15 * 1, 'to': 30 * 1})
    def deposit(self, amount_interval, account, deposit_token, min_native_balance, chain):
        contract = ZeroLendLiqContract(get_contracts_for_chain(chain)['ZEROLEND'], self.web3)

        zerolend_token = Erc20Token(chain, deposit_token, self.web3)

        amount = interval_to_eth_balance(amount_interval, account, chain, self.web3)

        if zerolend_token.allowance(account, contract.contract_address) < zerolend_token.balance(account).wei:
            zerolend_token.approve(account, contract.contract_address)

        contract.create_lock(account, amount)




from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import l0_evm_transaction
from sybil_engine.utils.file_loader import load_abi

from web3_wizzard_lib.core.modules.stargate.l0_data import l0_chain_ids

abi = load_abi("resources/abi/coredao_bridge.json")


class CoreDaoFromContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @l0_evm_transaction
    def bridge(self, account, amount, token, fee, to_chain):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = fee

        contract_txn = self.contract.functions.bridge(
            token,
            l0_chain_ids[to_chain],
            amount.wei,
            account.address,
            False,
            (
                account.address,
                '0x0000000000000000000000000000000000000000'
            ),
            b''
        ).build_transaction(txn_params)

        return contract_txn

    def estimate_bridge_fee(self, to_chain):
        return self.contract.functions.estimateBridgeFee(
            l0_chain_ids[to_chain],
            False,
            b''
        ).call()[0]



from sybil_engine.contract.send import Send
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.utils.sub_module import SubModule


class BitAvatar(SubModule):
    module_name = 'BITAVATAR'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['BITAVATAR']

        send = Send(None, web3)
        send.send_to_wallet(
            account, contract_address, NativeBalance(0, chain, "ETH"), "0x183ff085"
        )

    def log(self):
        return "BITAVATAR CHECK IN"



import random

from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import interval_to_eth_balance, from_eth_to_wei, verify_balance
from sybil_engine.module.module import Order, RepeatableModule
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.utils import ConfigurationException, interval_to_round
from sybil_engine.utils.validation_utils import validate_amount_interval, validate_interval
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.basiliskcontract import BasiliskContract
from web3_wizzard_lib.core.contract.eralendcontract import EraLendContract
from web3_wizzard_lib.core.contract.reactor_fusion_contract import ReactorFusionContract
from web3_wizzard_lib.core.modules.bank.layerbank import LayerBank
from web3_wizzard_lib.core.modules.bank.aave import Aave


class Lending(RepeatableModule):
    module_name = 'LENDING'
    module_config = 'lending_config'
    random_order = Order.RANDOM

    @RepeatableModule.repeatable_log
    def execute(self, lending_apps, amount: float, action: str, withdraw_sleep_interval: dict, chain,
                account: AppAccount):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        lending_app_name = random.choice(lending_apps)
        lending_app = self.get_lending_app(lending_app_name, chain, web3)

        verify_balance(self.min_native_balance, chain_instance, account, web3)

        if action == 'DEPOSIT':
            self.deposit(account, amount, lending_app, lending_app_name, chain, web3)
        elif action == 'WITHDRAW':
            self.withdraw(account, amount, chain, lending_app, lending_app_name)
        elif action == 'WITHDRAW_ALL':
            for app_name, app in self.lending_apps_with_deposit(account, chain, web3):
                self.withdraw(account, amount, chain, app, app_name)
        else:
            raise ConfigurationException("Unsupported action")

    def deposit(self, account, amount, lending_app, lending_app_name, chain, web3):
        amount_balance = interval_to_eth_balance(amount, account, chain, web3)

        if amount == 'all_balance':
            amount_balance = amount_balance.minus(self.min_native_balance)

        logger.info(f"Make deposit on {lending_app_name} | {amount_balance}")

        lending_app.mint(account, amount_balance.wei)

    def withdraw(self, account, amount, chain, lending_app, lending_app_name):
        deposit_balance = NativeBalance(lending_app.get_deposit_amount(account), chain, 'ETH')

        if deposit_balance.wei < 1000000000000:
            raise EmptyDepositException(f"[{account.address}] Deposit balance is 0, skip")

        logger.info(f"Deposit balance: {deposit_balance}")

        if amount == 'all_balance':
            withdraw_wei = deposit_balance.wei
        else:
            balance = NativeBalance(from_eth_to_wei(interval_to_round(amount)), chain, 'ETH')
            amount_balance = NativeBalance(int(balance.wei // 10000) * 10000, chain, balance.token)
            withdraw_wei = amount_balance.wei

            if deposit_balance.wei < withdraw_wei:
                logger.info(
                    f"Deposit {deposit_balance} < withdraw amount {amount_balance}"
                    f", withdrawing all deposit")
                withdraw_wei = deposit_balance.wei

        logger.info(f"Withdraw from {lending_app_name} | {deposit_balance}")

        lending_app.redeem_underlying(account, withdraw_wei)

    def get_lending_app(self, lending_app, chain, web3):
        lending_apps = self.get_lending_apps(chain, web3)

        if lending_app not in lending_apps.keys():
            raise ConfigurationException(f"{lending_app} not supported in {chain}. Supported lending: {list(lending_apps.keys())} supported")

        return lending_apps[lending_app]

    def get_lending_apps(self, chain, web3):
        contracts = get_contracts_for_chain(chain)

        apps = {
            'BASILISK_LANDING': BasiliskContract,
            'REACTORFUSION_LANDING': ReactorFusionContract,
            'ERALEND': EraLendContract,
            'AAVE': Aave,
            'LAYERBANK': LayerBank
        }

        return {k: v(contracts[k], web3) for k, v in apps.items() if k in contracts}

    def lending_apps_with_deposit(self, account, chain, web3):
        with_deposit = []

        for lending_app_name, lending_app in self.get_lending_apps(chain, web3).items():
            if lending_app.get_deposit_amount(account) > 1000000000000:
                with_deposit.append((lending_app_name, lending_app))

        return with_deposit

    def log(self):
        return "LENDING"

    def parse_params(self, module_params):
        validate_amount_interval(module_params['amount'])
        validate_interval(module_params['withdraw_sleep_interval'])

        if 'chain' not in module_params:
            module_params['chain'] = 'ZKSYNC'

        return [
            module_params['lending_apps'],
            module_params['amount'],
            module_params['action'],
            module_params['withdraw_sleep_interval'],
            module_params['chain']
        ]


class EmptyDepositException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)



import requests


def debank_request(address):
    url = "https://pro-openapi.debank.com/v1/user/total_balance"
    params = {
        'id': address
    }
    headers = {
        'accept': 'application/json',
        'AccessKey': '6cd56a970242386fa2a57e380c39f00f10b31778'
    }
    response = requests.get(url, headers=headers, params=params)
    data = response.json()
    return data


def debank_total_balance(address):
    return debank_request(address).get('total_usd_value', 0)


class Bank:
    def supply(self, account, amount):
        pass

    def borrow(self, account, amount):
        pass

    def get_repay_borrow_amount(self, account):
        pass

    def repay_borrow(self, account, amount):
        pass

    def redeem(self, account, amount, token):
        pass

    def get_deposit_amount(self, account, token):
        pass



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/zerolend_liquidity.json")

class ZeroLendLiqContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def create_lock(self, account, amount):
        sender = account.address
        txn_params = self.build_generic_data(sender, False)

        return self.contract.functions.createLock(
            int(amount.wei * 0.99),
            31536000,
            True
        ).build_transaction(txn_params)



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NotEnoughERC20Balance
from sybil_engine.domain.balance.balance_utils import interval_to_erc20_balance, get_native_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.retry import retry
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.coredao_contract import CoreDaoContract
from web3_wizzard_lib.core.contract.coredao_from_contract import CoreDaoFromContract
from web3_wizzard_lib.core.modules.bridge.bridge import Bridge


class CoreDaoBridgeSubModule(Bridge):
    supported_chains = ['POLYGON', 'COREDAO']

    @retry(max_attempts=5, retry_interval={'from': 60 * 15, 'to': 60 * 25})
    def execute(self, bridge_amount_interval, from_chain, to_chain, token, min_native_balance, account):
        chain_instance = get_chain_instance(from_chain)
        web3 = init_web3(chain_instance, account.proxy)

        erc20_token = Erc20Token(from_chain, token, web3)

        amount = interval_to_erc20_balance(bridge_amount_interval, account, token, from_chain, web3)

        logger.info(f"Native balance: {get_native_balance(account, web3, chain_instance)}")
        logger.info(f"Balance: {erc20_token.balance(account)}")

        logger.info(f"Bridge {from_chain + ' -> ' + to_chain} {amount}")

        if amount.wei == 0:
            raise NotEnoughERC20Balance(f"Can't swap zero balance")

        coredao_contract = get_contracts_for_chain(from_chain)['COREDAO_BRIDGE']

        if erc20_token.allowance(account, coredao_contract) < amount.wei:
            erc20_token.approve(account, coredao_contract)

        if from_chain == 'COREDAO':
            coredao = CoreDaoFromContract(coredao_contract, web3)
            fee = coredao.estimate_bridge_fee(to_chain)
        else:
            coredao = CoreDaoContract(coredao_contract, web3)
            fee = coredao.estimate_bridge_fee()

        coredao.bridge(account, amount, erc20_token.erc20_contract.contract_address, fee, to_chain)



from loguru import logger
from sybil_engine.contract.weth import WETH
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.balance import NotEnoughNativeBalance
from sybil_engine.domain.balance.balance_utils import interval_to_eth_balance, verify_balance, interval_to_weth_balance
from sybil_engine.module.module import Module
from sybil_engine.utils.utils import ConfigurationException
from sybil_engine.utils.validation_utils import validate_amount_interval
from sybil_engine.utils.web3_utils import init_web3


class Wrapping(Module):
    module_name = 'WRAPPING'
    module_config = 'wrap_config'
    allowed_chains = ['ZKSYNC', 'LINEA', 'BASE', 'SCROLL', 'ARBITRUM']
    sleep_after_var = True

    def execute(self, wrapping_chain, swap_amount_interval, action, account):
        self.sleep_after_var = True
        chain_instance = get_chain_instance(wrapping_chain)
        web3 = init_web3(chain_instance, account.proxy)

        weth_contract = get_tokens_for_chain(chain_instance['chain'])['WETH']
        weth = WETH(weth_contract, web3)

        if action == 'WRAP':
            amount_to_wrap = interval_to_eth_balance(swap_amount_interval, account, chain_instance['chain'], web3)

            logger.info(f"{action} {str(amount_to_wrap)}")

            native_balance = verify_balance(self.min_native_balance, chain_instance, account, web3)

            if swap_amount_interval == 'all_balance':
                amount_to_wrap = amount_to_wrap.minus(self.min_native_balance)

            if amount_to_wrap.wei > native_balance.wei:
                raise NotEnoughNativeBalance(
                    f"Account balance {native_balance} < {amount_to_wrap} amount to wrap.")

            weth.deposit(account, amount_to_wrap)
        elif action == 'UNWRAP':
            if swap_amount_interval == '':
                swap_amount_interval = 'all_balance'

            amount_to_unwrap = interval_to_weth_balance(
                swap_amount_interval,
                account,
                chain_instance['chain'],
                web3
            )

            if amount_to_unwrap.wei < 1000:
                logger.info(f"WETH balance is {amount_to_unwrap}, skip unwrapping")
                self.sleep_after_var = False
                return

            logger.info(f"{action} {amount_to_unwrap}")

            weth.withdraw(account, amount_to_unwrap)
        else:
            raise ConfigurationException(f'Wrong Action {action}')

    def parse_params(self, module_params):
        self.validate_supported_chain(module_params['chain'])
        validate_amount_interval(module_params['amount_interval'])

        return module_params['chain'], module_params['amount_interval'], module_params['action']

    def log(self):
        return "WRAPPING"

    def sleep_after(self):
        return self.sleep_after_var



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.tokens import Erc20Token

from web3_wizzard_lib.core.contract.zerolend_contract import ZeroLendContract
from web3_wizzard_lib.core.modules.bank.bank import Bank


class ZeroLend(Bank):
    app_name = 'ZEROLEND'
    supported_chains = ['LINEA']

    def __init__(self, chain, web3):
        self.chain = chain
        self.web3 = web3
        self.contract = ZeroLendContract(get_contracts_for_chain(self.chain)['ZEROLEND_LEND'], self.web3)

    def supply(self, account, amount):
        logger.info(f"Deposit {amount} to ZEROLEND")

        self.contract.deposit_eth(account, amount.wei)

    def redeem(self, account, amount):
        erc20_token = Erc20Token('LINEA', get_tokens_for_chain(self.chain)['ZEROLEND_WETH'], self.web3)

        if erc20_token.allowance(account, self.contract.contract_address) < 100:
            erc20_token.approve(account, self.contract.contract_address)

        self.contract.withdraw_eth(account, amount.wei)

    def get_deposit_amount(self, account, token):
        erc20_token = Erc20Token('LINEA', get_tokens_for_chain(self.chain)['ZEROLEND_WETH'], self.web3)

        return erc20_token.balance(account)




from loguru import logger
from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/nidus.json")


class NidumContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def mint_nft(self, account, claim_data):
        txn_params = self.build_generic_data(account.address, False)

        token_ids = [9]
        nft_amounts = [1]
        nonce = 0

        message = claim_data['message']
        signature = claim_data['signature']

        return self.contract.functions.mintFromShadowBatch(
            _tokenID=token_ids,
            _nftAmountForMint=nft_amounts,
            _nonce=nonce,
            _msgForSign=message,  # The original hashed message (no changes needed here)
            _signature=signature  # The generated signature
        ).build_transaction(txn_params)

    @evm_transaction
    def burn(self, account):
        txn_params = self.build_generic_data(account.address, False)
        token_id = 9

        balance = self.contract.functions.balanceOf(account.address, token_id).call()
        if balance == 0:
            logger.error(f"[{account.app_id}][{account.address}] No nidum nft. Mint first. Skip module")
            return

        return self.contract.functions.burn(
            self.contract.address, 9, 1
        ).build_transaction(txn_params)



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import interval_to_eth_balance
from sybil_engine.domain.balance.tokens import Erc20Token

from web3_wizzard_lib.core.contract.syncswap_pool import SyncSwapPoolContract
from web3_wizzard_lib.core.contract.syncswap_classic_pool import SyncSwapClassicPool
from web3_wizzard_lib.core.contract.syncswap_router import SyncSwapRouter
from web3_wizzard_lib.core.modules.liquidity.pool import Pool


class SyncswapPool(Pool):
    module_name = 'SYNCSWAP_LIQUIDITY'
    pool_contract = 'SYNCSWAP'

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.syncswap_router = SyncSwapRouter(self.velocore_pool_contract_address, self.web3)

    def deposit(self, amount_interval, account, token, min_native_balance, chain):
        amount = interval_to_eth_balance(amount_interval, account, chain, self.web3)

        pool = self.get_pool(chain)
        pool_amount = self.get_pool_amount(account, chain, pool)

        if pool_amount.wei < 10000000000:
            logger.info(f"Deposit {amount}")
            self.syncswap_router.add_liquidity2(account, amount)
        else:
            logger.info("This account already has deposit, skip")

    def withdraw(self, account, token, chain):
        pool = self.get_pool(chain)
        pool_amount = self.get_pool_amount(account, chain, pool)

        if pool_amount.wei < 10000000000:
            logger.info(f"Withdraw amount is to small")
            return

        erc20_token = Erc20Token(chain, pool, self.web3)

        if erc20_token.allowance(account, self.syncswap_router.contract_address) < pool_amount.wei:
            erc20_token.approve(account, self.syncswap_router.contract_address)

        print(erc20_token.balance(account).wei)

        logger.info(f"Withdraw {pool_amount}")
        self.syncswap_router.burn_liquidity(account, pool, pool_amount)

    def get_pool_amount(self, account, chain, pool):
        return NativeBalance(
            SyncSwapPoolContract(pool, self.web3).balanceOf(account),
            chain,
            'ETH'
        )

    def get_pool(self, chain):
        syncswap_classic_pool = SyncSwapClassicPool(get_contracts_for_chain(chain)["SYNCSWAP_CLASSIC_POOL_FACTORY"], self.web3)
        return syncswap_classic_pool.get_pool(get_tokens_for_chain(chain)['ETH'], get_tokens_for_chain(chain)['USDC'])



import requests
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.townstory_bonus_contract import TownstoryBonusContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class TownstoryBonus(SubModule):
    module_name = 'TOWNSTORY_BONUS'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['TOWNSTORY_BONUS']

        townstory_bonus_contract = TownstoryBonusContract(contract_address, web3)

        signature, deadline = get_travelbag_signature(account.address, account.proxy)

        townstory_bonus_contract.claim_linea_travelbag(account, signature, deadline)

    def log(self):
        return "CREATE TOWNSTORY BONUS"


def get_travelbag_signature(wallet, proxy):
    proxies = {'http': proxy,
               'https': proxy}
    url = 'https://townstory.io//api'
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        "Content-Length": "70",
        "Accept": "*/*",
        "Origin": "https://townstory.io",
        "Referer": "https://townstory.io/linea"

    }
    data = {"action": "getLineaSign", "address": f'{wallet.lower()}'}

    r = requests.post(url, data=data, headers=headers, proxies=proxies)
    res = r.json()
    return res['signature'], res['deadline']



from datetime import datetime

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

swap_method_id = 'a8c9ed67'
abi = load_abi("resources/abi/horizondex.json")


class HorizonDexRouter(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def multicall(self, account, amount_to_swap, amount_out_min, token_in_address, token_out_address):
        txn_params = self.build_generic_data(account.address, set_contract_address=False)

        if amount_to_swap.token == 'ETH':
            txn_params['value'] = amount_to_swap.wei  # amount in for ETH

        deadline_hex = decimal_to_padded_hexadecimal(int(datetime.now().timestamp() + 60 * 60 * 3), 64)
        amount_in = decimal_to_padded_hexadecimal(amount_to_swap.wei, 64)
        amount_out_min_hex = decimal_to_padded_hexadecimal(amount_out_min, 64)
        token_in_address_padded = pad_hex(token_in_address, 64).lower()
        token_out_address_padded = pad_hex(token_out_address, 64).lower()

        if amount_to_swap.chain == 'BASE':
            some_value = '000000000000000000000000000000000000000000000000000000000000000a'
        else:
            if amount_to_swap.token == 'USDC':
                some_value = '000000000000000000000000000000000000000000000000000000000000012c'
            else:
                some_value = '0000000000000000000000000000000000000000000000000000000000000028'

        swap = '0x' + swap_method_id + token_in_address_padded + token_out_address_padded + some_value + '0000000000000000000000000000000000000000000000000000000000000000' + deadline_hex + amount_in + amount_out_min_hex + '0000000000000000000000000000000000000000000000000000000000000000'
        unwrapETH = self.contract.encode_abi("unwrapWeth", args=(amount_out_min, account.address))

        return self.contract.functions.multicall([swap, unwrapETH]).build_transaction(txn_params)

    @evm_transaction
    def swap_exact_input_single(self, account, amount_to_swap, min_amount_out, token_in_address, token_out_address):
        sender = account.address

        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)
        txn_params = self.build_generic_data(sender, set_contract_address=False)
        txn_params['value'] = amount_to_swap.wei

        return self.contract.functions.swapExactInputSingle(
            (
                token_in_address,
                token_out_address,
                40,
                sender,
                deadline,
                amount_to_swap.wei,
                min_amount_out,
                0
            ),
        ).build_transaction(txn_params)

    @evm_transaction
    def swap_exact_input(self, account, min_amount_out, amount_to_swap, token_in_address, token_out_address):
        sender = account.address

        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)
        path = token_in_address + '00002842000000000000000000000000000000000000060003e8' + token_out_address

        txn_params = self.build_generic_data(sender)
        txn_params['value'] = amount_to_swap.wei
        txn_params['data'] = self.contract.encode_abi('swapExactInput', args=(
            (
                path,
                sender,
                deadline,
                amount_to_swap.wei,
                min_amount_out
            ),
        )
                                                     )
        return txn_params


def decimal_to_padded_hexadecimal(decimal_number, desired_length):
    hex_value = hex(decimal_number)[2:]

    return hex_value.zfill(desired_length)


def pad_hex(un_padded_hex, desired_length):
    return un_padded_hex[2:].zfill(desired_length)



import requests
from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import Erc20Balance
from sybil_engine.module.module import Module
from sybil_engine.utils.accumulator import add_accumulator_balance
from sybil_engine.utils.utils import AccountException
from sybil_engine.utils.web3_utils import init_web3
from web3 import Web3

from web3_wizzard_lib.core.contract.rage_claim import RageClaimer


class Rage(Module):
    module_name = 'RAGE_WITHDRAW'
    module_config = None

    def execute(self, account, chain='ARBITRUM'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain_instance['chain'])['RAGE_CLAIM']
        rage = RageClaimer(contract_address, web3)

        amount, index, merkle_proof = self.get_data(account)
        usdc_amount = int(amount, 16)

        usdc = Erc20Balance(usdc_amount, chain, 'USDC')

        logger.info(f"Withdraw {usdc.log_line()}")

        bytes32_array = [Web3.to_bytes(hexstr=hex_string[2:]) for hex_string in merkle_proof]

        rage.claim(account, usdc_amount, index, bytes32_array)
        add_accumulator_balance("Total rage withdraw", usdc_amount)

    def get_data(self, account):
        # Make the GET request
        url = f"https://www.app.rage.trade/api/merkel-proof?address={account.address}"

        response = requests.get(url)

        # Check if the request was successful (status code 200)
        if response.status_code == 200:
            # Parse the JSON response
            claim_data = response.json()["claim"]

            return claim_data["amount"], claim_data["index"], claim_data["proof"]
        else:
            raise AccountException(f"HTTP request failed with status code {response.status_code}")

    def log(self):
        return "RAGE CLAIMER"



from loguru import logger

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3_wizzard_lib.core.contract.horizondex_router import decimal_to_padded_hexadecimal

abi = load_abi("resources/abi/compound_v3_bulker.json")


class CompoundV3BulkerContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def invoke(self, account, amount, actions):
        hex_amount = decimal_to_padded_hexadecimal(int(amount * 0.98), 32)
        data = f'0x000000000000000000000000b2f97c1bd3bf02f5e74d13f02e3e26f93d77ce44000000000000000000000000ef0f48dadd0abe4f99b4c14862df303ba956bd1300000000000000000000000000000000{hex_amount}'

        txn = self.contract.functions.invoke(
            [hex_to_bytes(actions)],
            [hex_to_bytes(data)]
        )

        contract_txn = txn.build_transaction(self.build_generic_data(account.address, False))

        return contract_txn

def hex_to_bytes(hex_string):
    """
    Convert a hexadecimal string to bytes.

    Args:
        hex_string (str): The hexadecimal string, with or without '0x' prefix

    Returns:
        bytes: The converted bytes object
    """
    # Remove '0x' prefix if it exists
    if hex_string.startswith('0x'):
        hex_string = hex_string[2:]

    # Convert hex string to bytes
    return bytes.fromhex(hex_string)



from loguru import logger
from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/nomis_attest.json")


class NomisAttestContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def attest(self,
               account,
               schema,
               expirationTime,
               revocable,
               tokenId,
               updated,
               value,
               chainId,
               calcModel,
               validationPayload
        ):
        txn_params = self.build_generic_data(account.address, False)
        txn_params['gasPrice'] = int(self.web3.eth.gas_price * 2)
        return self.contract.functions.attestNomisScoreSimple(
            schema,
            expirationTime,
            revocable,
            tokenId,
            updated,
            value,
            chainId,
            calcModel,
            validationPayload
        ).build_transaction(txn_params)




import requests
from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.nomis_contract import NomisContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class Nomis(SubModule):
    module_name = 'NOMIS'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['NOMIS']

        response = requests.get(f'https://bg.nomis.cc/api/proxy/linea/wallet/{account.address}/score?scoreType=0&calculationModel=14&UseTokenLists=false&nonce=0&deadline=1790647549&GetCyberConnectProtocolData=false&prepareToMint=true&MintChain=0&ShouldGetReferrerCode=false&disableProxy=true&referrerCode=nomis', timeout=10)

        signature = response.json()['data']['mintData']['signature']
        mintedScore = response.json()['data']['mintData']['mintedScore']
        deadline = response.json()['data']['mintData']['deadline']
        metadataUrl = response.json()['data']['mintData']['metadataUrl']
        chainId = response.json()['data']['mintData']['chainId']
        referralCode = response.json()['data']['mintData']['referralCode']
        referrerCode = response.json()['data']['mintData']['referrerCode']
        calculationModel = response.json()['data']['mintData']['calculationModel']
        onftMetadataUrl = response.json()['data']['mintData']['onftMetadataUrl']

        nomis_contract = NomisContract(contract_address, web3)

        if mintedScore < 3000:
            formatted_score = "{:.2f}".format(mintedScore / 100)
            logger.info(f"Minted score is  {formatted_score}, minimum 30 is required, skip account")
            return

        nomis_contract.set_score(
            account,
            signature,
            mintedScore,
            calculationModel,
            deadline,
            metadataUrl,
            chainId,
            referralCode,
            referrerCode,
            onftMetadataUrl
        )

    def log(self):
        return "NOMIS POH"


def get_attest_data_media(token_auth):
    url = 'https://mp.trustalabs.ai/accounts/attest_calldata?attest_type=media'
    headers = {'Authorization': f'TOKEN {token_auth}', 'Accept': 'application/json'}
    r = requests.get(url, headers=headers)
    if r.status_code == 200:
        res = [r.json()]
        if res[0]['code'] == 0:
            txn_calldata = res[0]['data']
            return txn_calldata



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/reactor_fusion.json")


class ReactorFusionContract(Contract):

    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def get_deposit_amount(self, account):
        return self.contract.functions.balanceOf(account.address).call()

    @evm_transaction
    def mint(self, account, amount_wei: int):
        tx_data = self.build_generic_data(account.address)

        tx_data['data'] = self.contract.encode_abi('mint', args=())
        tx_data['gas'] = int(self.web3.eth.estimate_gas(tx_data))
        tx_data['value'] = amount_wei

        return tx_data

    @evm_transaction
    def redeem_underlying(self, account, amount):
        tx_data = self.build_generic_data(account.address, False)

        return self.contract.functions.redeem(amount).build_transaction(tx_data)

        # if amount > 0:
        #     logger.info(
        #         f"[{self.account_id}][{self.address}] Make withdraw from ReactorFusion | " +
        #         f"{Web3.from_wei(amount, 'ether')} ETH"
        #     )
        #
        #
        # else:
        #     logger.error(f"[{self.account_id}][{self.address}] Deposit not found")

    # @retry
    # @check_gas
    # async def enable_collateral(self):
    #     logger.info(f"[{self.account_id}][{self.address}] Enable collateral on ReactorFusion")
    #
    #     contract = self.get_contract(REACTORFUSION_CONTRACTS["collateral"], REACTORFUSION_ABI)
    #
    #     tx_data = await self.get_tx_data()
    #
    #     transaction = await contract.functions.enterMarkets(
    #         [Web3.to_checksum_address(REACTORFUSION_CONTRACTS["landing"])]
    #     ).build_transaction(tx_data)
    #
    #     signed_txn = await self.sign(transaction)
    #
    #     txn_hash = await self.send_raw_transaction(signed_txn)
    #
    #     await self.wait_until_tx_finished(txn_hash.hex())
    #
    # @retry
    # @check_gas
    # async def disable_collateral(self):
    #     logger.info(f"[{self.account_id}][{self.address}] Disable collateral on ReactorFusion")
    #
    #     contract = self.get_contract(REACTORFUSION_CONTRACTS["collateral"], REACTORFUSION_ABI)
    #
    #     tx_data = await self.get_tx_data()
    #
    #     transaction = await contract.functions.exitMarket(
    #         Web3.to_checksum_address(REACTORFUSION_CONTRACTS["landing"])
    #     ).build_transaction(tx_data)
    #
    #     signed_txn = await self.sign(transaction)
    #
    #     txn_hash = await self.send_raw_transaction(signed_txn)
    #
    #     await self.wait_until_tx_finished(txn_hash.hex())



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from eth_abi import encode

abi = load_abi("resources/abi/frog_war_contract.json")


class FrogWarContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def claim(self, account):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = 100000000000000

        return self.contract.functions.claim(
            account.address,
            1,
            1,
            '0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE',
            100000000000000,
            (
                ['0x0000000000000000000000000000000000000000000000000000000000000000'],
                # Note the brackets indicating an array
                1,
                0,
                '0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE'
            ),
            '0x'
        ).build_transaction(txn_params)

    @evm_transaction
    def claim_bonus(self, account):
        txn_params = self.build_generic_data(account.address, False)

        _receiver = account.address
        _tokenId = 6
        _quantity = 1
        _currency = '0x21d624c846725ABe1e1e7d662E9fB274999009Aa'
        _pricePerToken = 0

        return self.contract.functions.claim(
            _receiver, _tokenId, _quantity, _currency, _pricePerToken,
            [[encode(['bytes32'], [b''])], 1, _pricePerToken, _currency], b''
        ).build_transaction(txn_params)



from sybil_engine.contract.weth import WETH
from sybil_engine.data.networks import get_ids_chain
from sybil_engine.data.tokens import get_tokens_for_chain

from web3_wizzard_lib.core.contract.basiliskcontract import BasiliskContract
from web3_wizzard_lib.core.modules.bank.bank import Bank


class Basilisk(Bank):
    app_name = 'BASILISK_LANDING'
    supported_chains = ['ZKSYNC']

    def __init__(self, contract, web3):
        self.contract = BasiliskContract(contract, web3)

    def supply(self, account, amount):
        self.contract.mint(account, amount.wei)

    def redeem(self, account, withdraw):
        self.contract.redeem_underlying(account, withdraw.wei)

    def get_deposit_amount(self, account, token):
        weth_token = get_tokens_for_chain(get_ids_chain()[self.contract.web3.eth.chain_id])['BASILISK_LANDING']

        return WETH(weth_token, self.contract.web3).balance_of(account)



from sybil_engine.domain.dex import Dex

from web3_wizzard_lib.core.contract.zebra_contract import ZebraContract


class Zebra(Dex):
    dex_name = 'zebra'
    swap_contract = "ZEBRA"
    supported_chains = ['SCROLL']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.weth_address = self.tokens['WETH']
        zebra_contract = self.chain_contracts[self.swap_contract]
        self.zebra = ZebraContract(zebra_contract, self.web3)

    def swap_native_for_token(self, account, amount_to_swap, slippage, to_token_address):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, self.tokens['ETH'], to_token_address) * slippage)

        args = [account, amount_to_swap, self.tokens['ETH'], to_token_address, amount_out_min]
        func = self.zebra.swap_to_token
        return args, func

    def swap_token_for_native(self, account, amount_to_swap, from_token_address, slippage):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, from_token_address, self.weth_address) * slippage)

        args = [account, amount_to_swap, from_token_address, self.tokens['ETH'], amount_out_min]
        func = self.zebra.swap_to_eth
        return args, func

    def get_amount_out_min(self, amount_to_swap, from_token_address, to_token_address):
        return self.zebra.get_min_amount_out(amount_to_swap, from_token_address, to_token_address)



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/abbysworld.json")


class AlienSwapContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def purchase(self, account, amount):
        txn_params = self.build_generic_data(account.address, True)

        txn_params['value'] = 100000000000000

        txn_params['data'] = '0xefef39a10000000000000000000000000000000000000000000000000000000000000001'
        txn_params['gasPrice'] = self.web3.eth.gas_price

        return txn_params



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.domain.balance.balance import NotEnoughNativeBalance
from sybil_engine.domain.balance.balance_utils import from_wei_to_eth, from_eth_to_wei
from sybil_engine.utils.utils import interval_to_round

from web3_wizzard_lib.core.contract.stargate_router_eth import StargateRouterEth
from web3_wizzard_lib.core.modules.stargate.stargate_gas_checker import get_native_fee_balance


def bridge_eth(native_without_min, account, bridge_amount_interval, from_chain_instance, to_chain_instance, web3):
    contracts = get_contracts_for_chain(from_chain_instance['chain'])

    native_fee = get_native_fee_balance(
        'ETH',
        native_without_min,
        from_chain_instance,
        to_chain_instance,
        account,
        web3
    )

    if native_fee.wei > native_without_min.wei:
        raise NotEnoughNativeBalance(
            f"Native balance ({native_without_min}) "
            f"in {native_without_min.chain} is lower than required fee ({native_fee}",
            from_chain_instance['chain']
        )

    if bridge_amount_interval == 'all_balance':
        value_wei = native_without_min.wei

        if value_wei < 0:
            raise Exception('Not enough balance')

        amount_to_bridge_wei = value_wei - native_fee.wei
        amount_to_bridge_eth = from_wei_to_eth(amount_to_bridge_wei)
    else:
        amount_to_bridge_eth = interval_to_round(bridge_amount_interval)
        amount_to_bridge_wei = from_eth_to_wei(amount_to_bridge_eth)

        value_wei = amount_to_bridge_wei + native_fee.wei

    value_wei = value_wei
    value_eth = from_wei_to_eth(value_wei)

    if value_wei > native_without_min.wei:
        raise NotEnoughNativeBalance(
            f"The account balance ({native_without_min}) < bridging amount ({value_eth}{native_without_min.token}).")

    logger.info(f"Bridging: {amount_to_bridge_eth} {native_without_min.token}")
    logger.info(f"Trying to bridge with value: {value_eth}{native_without_min.token}")

    stargate_router_eth_address = contracts["STARGATE_ROUTER_ETH"]
    stargate_router = StargateRouterEth(stargate_router_eth_address, web3)

    args = [account, value_wei, amount_to_bridge_wei, to_chain_instance]

    stargate_router.swap_eth(*args)



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/bilinear.json")


class BilinearContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def claim(self, account):
        txn_params = self.build_generic_data(account.address, False)

        return self.contract.functions.claim(
            [],
            ([], [1]),
            ([], [], []),
            0,
            '0x307b8d76e3a3ca16066982ae42768878ec07a1b9af2679f2f9b8a3c3db25e7d5'
        ).build_transaction(txn_params)



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction

from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/rage_claim.json")


class RageClaimer(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def claim(self, account, amount, index, merkle_proof):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.claim(
            index,
            account.address,
            amount,
            merkle_proof
        ).build_transaction(txn_params)

        return contract_txn



from loguru import logger
from web3 import Web3

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/cog_erc20.json")

MAX_ALLOWANCE = 115792089237316195423570985008687907853269984665640564039457584007913129639935


class CogErc20Contract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def approve(self, account, contract_on_approve):
        logger.info(f"Approving token")

        txn_params = self.build_generic_data(account.address, set_contract_address=False)

        return self.contract.functions.approve(
            Web3.to_checksum_address(contract_on_approve),
            MAX_ALLOWANCE
        ).build_transaction(txn_params)

    @evm_transaction
    def redeem(self, account, amount):
        txn_params = self.build_generic_data(account.address, set_contract_address=False)

        return self.contract.functions.redeem(amount).build_transaction(txn_params)

    def balance_of(self, account):
        return self.contract.functions.balanceOf(account.address).call()

    def max_redeem(self, account):
        return self.contract.functions.maxRedeem(account.address).call()

    def allowance(self, account, allowance_contract):
        return self.contract.functions.allowance(account.address, Web3.to_checksum_address(allowance_contract)).call()

    def decimals(self):
        return self.contract.functions.decimals().call()

    def symbol(self):
        return self.contract.functions.symbol().call()



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/new_rage_abi.json")


class NewRageContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def redeem(self, account, shares):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.redeem(
            shares,
            account.address,
            account.address,
        ).build_transaction(txn_params)

        return contract_txn

    def balance_of(self, account):
        return self.contract.functions.balanceOf(account.address).call()

    def convert_to_shares(self, amount):
        return self.contract.functions.convertToShares(amount).call()

    def max_redeem(self, account):
        return self.contract.functions.maxRedeem(account.address).call()



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/linea_ens_names.json")


class LineaEnsNamesContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)
        self.addr_contract = web3.eth.contract(
            abi='[{"inputs":[{"internalType":"bytes32","name":"node","type":"bytes32"},{"internalType":"uint256","name":"coinType","type":"uint256"},{"internalType":"bytes","name":"a","type":"bytes"}],"name":"setAddr","outputs":[],"stateMutability":"nonpayable","type":"function"},{"inputs":[{"internalType":"bytes32","name":"node","type":"bytes32"},{"internalType":"address","name":"a","type":"address"}],"name":"setAddr","outputs":[],"stateMutability":"nonpayable","type":"function"}]'
        )

    def available(self, linea_name):
        return self.contract.functions.available(linea_name).call()

    def make_commitment(self, account, linea_name, duration, secret, data):
        return self.contract.functions.makeCommitment(
            linea_name,
            account.address,
            duration,
            secret,
            "0x86c5AED9F27837074612288610fB98ccC1733126",
            [data],
            True,
            0,
        ).call().hex()

    def redeemed(self, account):
        return self.contract.functions.redeemed(account.address).call()

    @evm_transaction
    def register_poh(self, account, linea_name, duration, secret, data, poh_signature):
        txn_params = self.build_generic_data(account.address, set_contract_address=False)

        return self.contract.functions.registerPoh(
            linea_name,
            account.address,
            duration,
            secret,
            "0x86c5AED9F27837074612288610fB98ccC1733126",
            [data],
            True,
            0,
            poh_signature["sign"]
        ).build_transaction(txn_params)

    @evm_transaction
    def commit(self, account, commitment):
        txn_params = self.build_generic_data(account.address, set_contract_address=False)

        return self.contract.functions.commit(
            "0x" + commitment
        ).build_transaction(txn_params)



from loguru import logger
from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/nomis.json")


class NomisContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def set_score(self,
                  account,
                  signature,
                  mintedScore,
                  calculationModel,
                  deadline,
                  metadataUrl,
                  chainId,
                  referralCode,
                  referrerCode,
                  onftMetadataUrl
                  ):
        txn_params = self.build_generic_data(account.address, False)
        txn_params['value'] = Web3.to_wei(0.001, 'ether')
        return self.contract.functions.setScore(
            signature,
            mintedScore,
            calculationModel,
            deadline,
            metadataUrl,
            chainId,
            referralCode,
            referrerCode,
            onftMetadataUrl
        ).build_transaction(txn_params)




from datetime import datetime

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.data.networks import get_ids_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/mute_router.json")


class MuteRouter(Contract):
    def __init__(self, contract_address, web3):
        tokens = get_tokens_for_chain(get_ids_chain()[web3.eth.chain_id])
        self.weth_address = tokens['WETH']
        self.usdc_address = tokens['USDC']
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def swap_exact_eth_for_tokens_supporting_fee_on_transfer_tokens(self, account, amount_to_swap, token_in_address,
                                                                    token_out_address, amount_out_min):
        sender = account.address

        path = [token_in_address, token_out_address]
        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)
        stable = [True, False] if token_out_address == self.usdc_address else [False, False]

        txn_params = self.build_generic_data(sender)

        txn_params['value'] = amount_to_swap.wei
        txn_params['data'] = self.contract.encode_abi('swapExactETHForTokensSupportingFeeOnTransferTokens', args=(
            amount_out_min,
            path,
            sender,
            deadline,
            stable
        )
                                                     )

        txn_params['gas'] = self.web3.eth.estimate_gas(txn_params)

        return txn_params

    @evm_transaction
    def swap_exact_tokens_for_eth_supporting_fee_on_transfer_tokens(self, account, amount_to_swap, token_in_address,
                                                                    token_out_address, amount_out_min):
        sender = account.address

        txn_params = self.build_generic_data(sender, False)

        path = [token_in_address, token_out_address]
        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)
        stable = [False, False]

        contract_txn = self.contract.functions.swapExactTokensForETHSupportingFeeOnTransferTokens(
            amount_to_swap.wei,
            amount_out_min,
            path,
            sender,
            deadline,
            stable
        ).build_transaction(txn_params)

        return contract_txn

    @evm_transaction
    def swap_exact_tokens_for_tokens_supporting_fee_on_transfer_tokens(self, account, amount_to_swap, token_in_address,
                                                                       token_out_address, amount_out_min):
        sender = account.address

        txn_params = self.build_generic_data(sender, False)

        path = [token_in_address, self.weth_address, token_out_address]
        deadline = int(datetime.now().timestamp() + 60 * 60 * 3)
        stable = [True, False, False] if token_in_address == self.usdc_address else [False, True, False]

        contract_txn = self.contract.functions.swapExactTokensForTokensSupportingFeeOnTransferTokens(
            amount_to_swap.wei,
            amount_out_min,
            path,
            sender,
            deadline,
            stable
        ).build_transaction(txn_params)

        return contract_txn

    def get_amount_out(self, amount_to_swap, token_in_address, token_out_address):
        return self.contract.functions.getAmountOut(
            amount_to_swap.wei,
            token_in_address,
            token_out_address
        ).call()[0]



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/yoddlo.json")


class YoddloContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def purchase(self, account):
        txn_params = self.build_generic_data(account.address, True)

        txn_params['value'] = 100000000000000
        txn_params['data'] = '0xfb89f3b1'

        return txn_params



from json import dumps
from time import time

import requests
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.townstory_contract import TownstoryContract
from web3_wizzard_lib.core.modules.nft.sign import sign_msg
from web3_wizzard_lib.core.utils.sub_module import SubModule


class Townstory(SubModule):
    module_name = 'TOWNSTORY'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['TOWNSTORY']

        message_text = get_message(account)
        message_signature = sign_msg(account, message_text, web3)
        txn_signature, deadline = get_txn_signature(account, message_signature)

        townstory_contract = TownstoryContract(contract_address, web3)

        townstory_contract.create_account_sign(account, txn_signature, deadline)

    def log(self):
        return "CREATE TOWNSTORY ACOUNT"


def get_txn_signature(account, message_signature):
    data = {"header": {"version": "1.0.1", "baseVersion": "1.0.0", "referer": ""},
            "transaction": {"func": "register.loginByWallet",
                            "params": {"hall": 0, "wallet": "metamask", "chain": "linea",
                                       "signature": message_signature, "address": account.address}}}
    json_data = dumps(data)
    headers = {'Content-Type': 'application/x-www-form-urlencoded'}
    url1 = 'https://aws-login.townstory.io/town-login/handler.php'
    r = requests.get(url1, data=json_data, headers=headers)
    response = r.json()
    if response['result'] != 'failed':
        txn_signature = r.json()['response']['signature']
        deadline = r.json()['response']['deadline']
        return txn_signature, deadline


def get_time_nonce():
    time_nonce = int(time() / 86400)
    return time_nonce


def get_address_line(address):
    address_line = (address[:19] + '...' + address[-18:]).lower()
    return address_line


def get_message(wallet):
    nonce = get_time_nonce()
    address_line = get_address_line(wallet.address)
    message = ('Welcome to Town Story! \n\n'
               'Click to sign in and accept the Town Story\n'
               'Terms of Service:\n'
               'https://townstory.io/\n\n'
               'This request will not trigger a blockchain\n'
               'transaction or cost any gas fees.\n\n'
               'Your authentication status will reset after\n'
               'each session.\n\n'
               'Wallet address:\n'
               f'{address_line}\n\n'
               f'Nonce: {nonce}')
    return message



from loguru import logger
from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance_utils import interval_to_erc20_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.module.module import Module
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.utils import AccountException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.zkdx_contract import ZKDXContract
from web3_wizzard_lib.core.contract.zkdx_data import ZKDXDataContract


class ZKDX(Module):
    module_name = 'ZKDX'
    module_config = 'zkdx_config'

    def execute(self, action, amount_interval, account: AppAccount, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        zkdx_contract = get_contracts_for_chain(chain)['ZKDX']
        zkdx_data_contract = get_contracts_for_chain(chain)['ZKDX_DATA']

        zkdx = ZKDXContract(zkdx_contract, web3)

        if action == 'DEPOSIT':
            amount = interval_to_erc20_balance(amount_interval, account, 'USDC', chain, web3)

            erc20 = Erc20Token(chain, 'USDC', web3)

            if erc20.allowance(account, zkdx_contract) < erc20.balance(account).wei:
                erc20.approve(account, zkdx_contract)

            logger.info(f"Deposit {amount} to ZKDX")
            zkdx.swap_to_zkdx(account, amount)
        if action == 'WITHDRAW':
            zkdx_data = ZKDXDataContract(zkdx_data_contract, web3)
            balance = zkdx_data.get_balance(account)[0]

            if balance == 0:
                raise AccountException("ZKDX Balance is 0, skip")

            logger.info(f"WITHDRAW {balance / 10 ** 18} ZUSD to ZKDX")
            zkdx.swap_from_zkdx(account, balance)
        else:
            raise Exception(f"{action} action not supported")

    def log(self):
        return "ZKDX"

    def parse_params(self, module_params):
        return [
            module_params['action'],
            module_params['amount_interval']
        ]


class EmptyDepositException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)



from sybil_engine.domain.dex import Dex

from web3_wizzard_lib.core.contract.horizondex_router import HorizonDexRouter
from ...contract.horizondex_quoter import HorizonDexQuoter


class Horizondex(Dex):
    dex_name = 'horizondex'
    swap_contract = 'HORIZONDEX'
    supported_chains = ['BASE', 'LINEA']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.weth_address = self.tokens['WETH']
        self.horizondex_router = HorizonDexRouter(self.chain_contracts[self.swap_contract], self.web3)

    def swap_token_for_native(self, account, amount_to_swap, from_token_address, slippage):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, from_token_address, self.weth_address) * slippage)
        args = [account, amount_to_swap, amount_out_min, from_token_address, self.weth_address]
        func = self.horizondex_router.multicall

        return args, func

    def swap_native_for_token(self, account, amount_to_swap, slippage, to_token_address):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, self.weth_address, to_token_address) * slippage)
        args = [account, amount_to_swap, amount_out_min, self.weth_address, to_token_address]
        func = self.horizondex_router.swap_exact_input_single

        return args, func

    def swap_token_for_token(self, account, amount_to_swap, slippage, from_token_address, to_token_address):
        raise Exception("Not supported yet")

    def get_amount_out_min(self, amount_to_swap, from_token_address, to_token_address):
        horizondex_quoter_contract = self.chain_contracts["HORIZONDEX_QUOTER"]
        horizondex_quoter = HorizonDexQuoter(horizondex_quoter_contract, self.web3)

        return horizondex_quoter.quote_exact_input_single(amount_to_swap.wei, from_token_address, to_token_address)



from eth_account.messages import encode_defunct
from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.module.module import Module
from sybil_engine.utils.file_loader import load_abi
from sybil_engine.utils.utils import AccountException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.intract.intract_api import IntractAPI
from web3_wizzard_lib.core.modules.intract.utils import TgReport, WindowName, make_text_border

VERIFY_DATA = load_abi("resources/verify_data.json")

class IntractClaim(Module):
    module_name = 'INTRACT_CLAIM'
    module_config = None

    def execute(self, task_id, account, compaign_id='66bb5618c8ff56cba848ea8f', from_chain='LINEA'):
        windowname = WindowName()

        intract_api = IntractAPI(account)
        tg_report = TgReport()
        windowname.update_accs()

        stats = {}

        web3 = init_web3(get_chain_instance(from_chain), None)

        self.authorize(intract_api, compaign_id, account, web3)

        campaign_funcs = {
            "Linea LXP Latest Rush": self.linea_lxp,
        }
        intract_api.set_primary_task_identity(account)
        intract_api.activate_journey(compaign_id)
        linea_xp = intract_api.get_user().get("totalXp") or "0"

        if task_id == 'SKIP_CHECK':
            if int(linea_xp) == 100:
                raise AccountException("Skip account tasks where already claimed")
            else:
                return

        for task in intract_api.active_tasks:
            if task in list(campaign_funcs.keys()):
                campaign_funcs[task](task, task_id, intract_api)
            else:
                logger.info(f' > Task "{task}" will appear soon...')

        account_data = intract_api.get_super_user()
        linea_xp = intract_api.get_user().get("totalXp") or "0"

        logger.info('')  # for clearly logs
        tg_report.update_logs('<i>Statistics</i>')
        logger.info('')
        text = f'Achievements: {account_data["achieves"]}\n' \
               f'GM Streak: {account_data["streak"]}\n' \
               f'Gems: {account_data["gems"]}\n' \
               f'XP: {account_data["xp"]}\n' \
               f'Linea XP: {linea_xp}'
        new_text = make_text_border(text=text)
        for string in new_text.split('\n'): logger.success(string)

        stats.update({
            'linea_xp': linea_xp,
            'xp': account_data["xp"],
            'gm_streak': account_data["streak"],
            'achievements': account_data["achieves"],
            'gems': account_data["gems"],
        })

    def authorize(self, browser, compaign_id, account, web3):
        nonce = browser.intract_get_nonce(address=account.address)
        text = f'Nonce: {nonce}'
        signature = f'0x{web3.eth.account.sign_message(encode_defunct(text=text), private_key=account.key).signature.hex()}'
        browser.intract_login(address=account.address, signature=signature)
        browser.intract_get_tasks(compaign_id)

        browser.set_wallet(address=account.address)
        #browser.claim_achievements()

    def linea_lxp(self, task_name: str, task_id, intract_api, week=0):
        logger.info(f' > Completing "{task_name}"...')

        for task in VERIFY_DATA['campaigns'][str(week)]['quest']['tasks']:
            if task_id is not None:
                if task['_id'] == task_id:
                    intract_api.verify_quest(task)
            else:
                intract_api.verify_quest(task)

    def log(self):
        return "INTRACT CLAIM"

    def parse_params(self, module_params):
        if 'taskID' not in module_params:
            module_params['taskID'] = None

        return [
            module_params['taskID']
        ]



import random
import webbrowser

from loguru import logger
from sybil_engine.module.module import Module
from sybil_engine.utils.accumulator import add_accumulator, get_value

from web3_wizzard_lib.core.utils.ai_utils import get_ai_chat

from web3_wizzard_lib.core.utils.module_memory import get_by_key, accumulate_by_key, remove_key, add_value
from web3_wizzard_lib.core.utils.statistic_utils import get_statistic_writer, statistic_date_string

APPEAL_ACCOUNTS = "APPEAL_ACCOUNTS"
APPEAL_ACCOUNTS_AMOUNT = "APPEAL_ACCOUNTS_AMOUNT"


class LineaAppeal(Module):
    module_name = 'LINEA_APPEAL'
    module_config = "linea_appeal_config"
    base_url = "https://docs.google.com/forms/d/e/1FAIpQLSfkbHzC1hZTy6u5R8S5i6wQ2xCyUZQjvucmlyChwg04fJIO5Q/viewform"

    def execute(self, token, accounts, statistic_write, ai_type, account):
        with open("resources/linea_appeal.txt") as f:
            self.linea_appeal_reason = f.read()

        with open("resources/ai_style_randomized.txt") as f:
            self.lines = f.readlines()

        with open("resources/ai_reason_randomized.txt") as f:
            self.reason_lines = f.readlines()

        add_accumulator("Acc Num", 1)

        statistics = get_statistic_writer()
        statistics.init_if_required(
            f"linea_appeal_{statistic_date_string}",
            ["#", "MainAddress", "GPT Answer"]
        )

        chat_gpt = get_ai_chat(ai_type, token)

        if get_by_key(APPEAL_ACCOUNTS_AMOUNT) is None:
            add_value(
                APPEAL_ACCOUNTS_AMOUNT,
                random.randint(accounts['from'], accounts['to'])
            )

        accumulate_by_key(
            APPEAL_ACCOUNTS, {
                "address": account.address,
            }
        )

        logger.info(f"Acc: {get_value("Acc Num")}")
        logger.info(f"Acc Amount: {get_value("Acc Amount")}")

        if (get_by_key(APPEAL_ACCOUNTS_AMOUNT) == len(get_by_key(APPEAL_ACCOUNTS))
                or get_value("Acc Num") == get_value("Acc Amount")):
            ai_style_randomized = random.choice(self.lines)
            selected_lines = random.sample(self.reason_lines, k=random.choice([2, 3]))

            reason = chat_gpt.ask(
                self.linea_appeal_reason
                    .replace("{random_style}", ai_style_randomized)
                    .replace("{random_reasons}", str(selected_lines))
            )
            logger.info(reason)

            statistics.write_row(
                statistic_date_string,
                [account.app_id, account.address, reason]
            )

            wallets = get_by_key(APPEAL_ACCOUNTS)
            address_list = [wallet["address"] for wallet in wallets]
            address_list.remove(account.address)
            self.open_appeal_form(account, "\n".join(address_list), reason)
            remove_key(APPEAL_ACCOUNTS)
            remove_key(APPEAL_ACCOUNTS_AMOUNT)

    def open_appeal_form(self, account, address_list, formatted_string):
        payload = {
            "entry.1292139045": account.address,
            "entry.1099559693": address_list.replace("\n", "%0A"),
            "entry.1296389817": formatted_string.replace("\n", "%0A")
        }
        query_string = "&".join(f"{key}={value}" for key, value in payload.items())
        pre_filled_url = f"{self.base_url}?{query_string}"
        print(f"Opening form: {pre_filled_url}")
        webbrowser.open(pre_filled_url)

    def parse_params(self, module_params):
        return (
            module_params['ai_token'],
            module_params['accounts'],
            module_params['write_mode'],
            module_params['ai_type']
        )



from sybil_engine.contract.send import Send
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.utils.sub_module import SubModule


class Efrogs(SubModule):
    module_name = 'LINEA_CULTURE_5'

    def execute(self, account: AppAccount, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        send = Send(None, web3)
        send.send_to_wallet(
            account,
            '0xf4AA97cDE2686Bc5ae2Ee934a8E5330B8B13Be64',
            NativeBalance(0, chain, "ETH"),
            "0x00000000"
        )

    def log(self):
        return "LINEA CULTURE 5 (EFROGS)"



from loguru import logger
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.domain.dex import Dex
from sybil_engine.utils.gas_utils import l1_gas_price
from sybil_engine.utils.utils import AccountException

from web3_wizzard_lib.core.contract.xy_finance_contract import XYSwapContract


class XyFinance(Dex):
    dex_name = 'xy.finance'
    swap_contract = 'XY_FINANCE'
    supported_chains = ['SCROLL']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.contract_address = self.chain_contracts[self.swap_contract]
        self.xy_swap_contract = XYSwapContract(self.contract_address, self.web3)

    @l1_gas_price
    def swap(self, amount_to_swap, from_token, to_token, slippage, account):
        if amount_to_swap.wei == 0:
            raise AccountException(f"{amount_to_swap.token} balance is 0")

        token_in_address = "0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE" if from_token == "ETH" else self.tokens[from_token]
        token_out_address = "0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE" if to_token == "ETH" else self.tokens[to_token]

        logger.info(f"Swap {amount_to_swap.log_line()}->{to_token} in {self.dex_name} ({self.chain_instance['chain']})")

        erc20_from_token = Erc20Token(self.chain_instance['chain'], amount_to_swap.token, self.web3)

        if amount_to_swap.token != 'ETH':
            if erc20_from_token.allowance(account, self.contract_address) < amount_to_swap.wei:
                erc20_from_token.approve(account, self.contract_address)

        slippage = int((1 - slippage) * 100)

        amount_out_min = self.get_amount_out_min(amount_to_swap, token_in_address, token_out_address, slippage)

        self.xy_swap_contract.swap(account, amount_to_swap, token_in_address, token_out_address, amount_out_min, slippage)

    def get_amount_out_min(self, amount_to_swap, token_in_address, token_out_address, slippage):
        return self.xy_swap_contract.get_quote(token_in_address, token_out_address, amount_to_swap, slippage)



from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance_utils import verify_balance
from sybil_engine.utils.retry import retry_self
from sybil_engine.utils.utils import AccountException
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.stargate.bridge_eth import bridge_eth
from web3_wizzard_lib.core.modules.stargate.bridge_tokens import bridge_usdc

MAX_ATTEMPTS = 5


class StargateBridge:
    def __init__(self, retry_interval):
        self.retry_interval = retry_interval

    def bridge(self, min_native_balance, account, bridge_amount_interval, token, from_chain, to_chain):
        from_chain_config = get_chain_instance(from_chain)
        to_chain_config = get_chain_instance(to_chain)
        web3 = init_web3(from_chain_config, account.proxy)

        logger.info(f"======= Bridging {token} {from_chain} ==> {to_chain} ::: {account.address} =======")

        self._bridge(account, bridge_amount_interval, from_chain_config, min_native_balance, to_chain_config, token,
                     web3)

    @retry_self(max_attempts=MAX_ATTEMPTS, expected_exception=Exception, throw_exception=AccountException)
    def _bridge(self, account, bridge_amount_interval, from_chain_config, min_native_balance, to_chain_config, token,
                web3):
        native_without_min = verify_balance(min_native_balance, from_chain_config, account, web3)

        if token == 'ETH':
            bridge_eth(native_without_min, account, bridge_amount_interval, from_chain_config, to_chain_config, web3)
        elif token == 'USDC':
            bridge_usdc(native_without_min, account, bridge_amount_interval, from_chain_config, to_chain_config, web3)
        else:
            raise ValueError(f"Unsupported bridge token: {token}, only ETH and USDC are supported")



from sybil_engine.domain.dex import Dex

from web3_wizzard_lib.core.contract.velocore_router import VelocoreRouter


class Velocore(Dex):
    dex_name = 'velocore'
    swap_contract = "VELOCORE_ROUTER"
    supported_chains = ['ZKSYNC']

    def __init__(self, chain_instance, web3):
        super().__init__(chain_instance, web3)
        self.weth_address = self.tokens['WETH']
        self.velocore_router = VelocoreRouter(self.chain_contracts[self.swap_contract], self.web3)

    def swap_native_for_token(self, account, amount_to_swap, slippage, to_token_address):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, self.weth_address, to_token_address) * slippage)

        args = [account, amount_to_swap, self.weth_address, to_token_address, amount_out_min]
        func = self.velocore_router.swap_exact_eth_for_tokens

        return args, func

    def get_amount_out_min(self, amount_to_swap, from_token_address, to_token_address):
        return self.velocore_router.get_amount_out(amount_to_swap, from_token_address, to_token_address)

    def swap_token_for_native(self, account, amount_to_swap, from_token_address, slippage):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, from_token_address, self.weth_address) * slippage)

        args = [account, amount_to_swap, from_token_address, self.weth_address, amount_out_min]
        func = self.velocore_router.swap_exact_tokens_for_eth
        return args, func

    def swap_token_for_token(self, account, amount_to_swap, slippage, from_token_address, to_token_address):
        amount_out_min = int(self.get_amount_out_min(amount_to_swap, from_token_address, to_token_address) * slippage)

        args = [account, amount_to_swap, from_token_address, to_token_address, amount_out_min]
        func = self.velocore_router.swap_exact_tokens_for_tokens

        return args, func



from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_ids_chain
from sybil_engine.data.tokens import get_tokens_for_chain
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.utils import ConfigurationException

from web3_wizzard_lib.core.contract.compound_v3 import CompoundV3Contract
from web3_wizzard_lib.core.contract.compund_v3_bulker import CompoundV3BulkerContract
from web3_wizzard_lib.core.modules.bank.bank import Bank


class CompoundV3(Bank):
    app_name = 'COMPOUND_V3'
    supported_chains = ['SCROLL']

    def __init__(self, chain, web3):
        compound_v3_address = get_contracts_for_chain(chain)[self.app_name]
        self.contract = CompoundV3Contract(compound_v3_address, web3)
        compound_v3_bulker_address = get_contracts_for_chain(chain)['COMPOUND_V3_BULKER']
        self.contract_bulker = CompoundV3BulkerContract(compound_v3_bulker_address, web3)

    def supply(self, account, amount):
        raise ConfigurationException("Only redeem supported for Compound V3")

    def get_repay_borrow_amount(self, account):
        return self.contract.borrow_balance_of(account)

    def repay_borrow(self, account, amount):
        erc20_token = Erc20Token(self.contract.chain_instance['chain'], "USDC", self.contract.web3)

        if erc20_token.allowance(account, self.contract.contract_address) < 100:
            erc20_token.approve(account, self.contract.contract_address)

        return self.contract.supply(account, amount, erc20_token.erc20_contract.contract_address)

    def redeem(self, account, amount, token):
        actions = '0x414354494f4e5f57495448445241575f4e41544956455f544f4b454e00000000'
        self.contract_bulker.invoke(account, amount, actions)

    def get_deposit_amount(self, account, token):
        token_address = get_tokens_for_chain(get_ids_chain()[self.contract.web3.eth.chain_id])[token]
        compound_v3_address = get_contracts_for_chain(get_ids_chain()[self.contract.web3.eth.chain_id])[self.app_name]

        return CompoundV3Contract(compound_v3_address, self.contract.web3).user_collateral(
            account,
            token_address
        )



import random

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.data.pairs import Pairs
from sybil_engine.domain.balance.balance_utils import verify_balance, amount_to_swap_from_interval
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.module.module import Order, RepeatableModule
from sybil_engine.utils.utils import interval_to_int, randomized_sleeping, SwapException
from sybil_engine.utils.validation_utils import validate_amount_interval_possible_empty, validate_token, \
    validate_interval, validate_dex_list
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.swap.swap_list import swap_facade


class WarmUp(RepeatableModule):
    module_name = 'WARMUP'
    module_config = 'warmup_config'
    allowed_chains = ['ZKSYNC', 'LINEA', 'BASE', 'SCROLL', 'ARBITRUM']
    random_order = Order.RANDOM
    repeat_conf = 'amount_of_warmups'

    @RepeatableModule.repeatable_log
    def execute(self, chain, swap_amount_interval, warm_token, amount_of_warmups, allowed_dex, pair_names: list,
                sell_tokens, sleep_interval, account):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        swap_amount = interval_to_int(amount_of_warmups)

        if len(pair_names) > 0:
            logger.info(f"Warmup pairs {pair_names}")

        pair = Pairs(swap_facade)

        pair_swaps = pair.get_warmup_pair_swaps(allowed_dex, chain, pair_names, swap_amount, warm_token)
        random.shuffle(pair_swaps)
        self._warm_up(chain_instance, pair_swaps, sell_tokens, sleep_interval, swap_amount_interval, account, web3)

    def _warm_up(self, chain_instance, pair_swaps, sell_tokens, sleep_interval, swap_amount_interval, account, web3):
        pair, swaps = random.choice(pair_swaps)
        random.shuffle(swaps)

        native_balance = verify_balance(self.min_native_balance, chain_instance, account, web3)
        buy_dex, sell_dex = swaps[0], swaps[-1]

        self._warm_up_pair(buy_dex, sell_dex, native_balance, swap_amount_interval, chain_instance, pair, sell_tokens,
                           sleep_interval, account, web3)

        randomized_sleeping(sleep_interval)

    def _warm_up_pair(self, buy_dex, sell_dex, native_balance, swap_amount_interval, chain_instance, pair, sell_tokens,
                      sleep_interval, account, web3):
        warm_up_token = pair['tokens'][0]
        token = pair['tokens'][1]

        amount_to_swap = amount_to_swap_from_interval(account, chain_instance['chain'], self.min_native_balance,
                                                      native_balance, swap_amount_interval, warm_up_token, web3)
        swap_facade.swap(account, amount_to_swap, chain_instance, pair, buy_dex, warm_up_token, token, web3)
        randomized_sleeping(sleep_interval)

        if sell_tokens:
            try:
                self.execute_sell(chain_instance, pair, sell_dex, account, web3)
            except SwapException as e:
                logger.error(e)
                self.execute_sell(chain_instance, pair, buy_dex, account, web3)

    def execute_sell(self, chain_instance, pair, sell_dex, account, web3):
        amount_to_swap = Erc20Token(chain_instance['chain'], pair['tokens'][1], web3).balance(account)
        swap_facade.swap(account, amount_to_swap, chain_instance, pair, sell_dex, amount_to_swap.token,
                         pair['tokens'][0], web3)

    def parse_params(self, module_params):
        self.validate_supported_chain(module_params['chain'])
        validate_amount_interval_possible_empty(module_params["swap_amount_interval"])
        validate_token(module_params["token"])
        validate_interval(module_params["amount_of_warmups"])
        validate_dex_list(module_params["allowed_dex"])
        validate_interval(module_params["pair_sleep_interval"])

        return (
            module_params["chain"],
            module_params["swap_amount_interval"],
            module_params["token"],
            module_params["amount_of_warmups"],
            module_params["allowed_dex"],
            module_params["warmup_pairs"],
            module_params["sell_tokens"],
            module_params["pair_sleep_interval"]
        )

    def log(self):
        return "WARM UP"

    def sleep_after(self):
        return False



import time

from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi
from web3 import Web3

abi = load_abi("resources/abi/zebra.json")


class ZebraContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def get_min_amount_out(self, amount, from_token: str, to_token: str):
        min_amount_out = self.contract.functions.getAmountsOut(
            amount.wei,
            [
                Web3.to_checksum_address(from_token),
                Web3.to_checksum_address(to_token)
            ]
        ).call()
        return min_amount_out[1]

    @evm_transaction
    def swap_to_token(self, account, amount, from_token: str, to_token: str, min_amount_out: int):
        tx_data = self.build_generic_data(
            account.address,
            False
        )

        tx_data['value'] = amount.wei

        deadline = int(time.time()) + 1000000

        contract_txn = self.contract.functions.swapExactETHForTokens(
            min_amount_out,
            [
                from_token,
                to_token,
            ],
            account.address,
            deadline
        ).build_transaction(tx_data)

        return contract_txn

    @evm_transaction
    def swap_to_eth(self, account, amount, from_token: str, to_token: str, min_amount_out: int):
        tx_data = self.build_generic_data(
            account.address,
            False
        )

        deadline = int(time.time()) + 1000000

        contract_txn = self.contract.functions.swapExactTokensForETH(
            amount.wei,
            min_amount_out,
            [
                from_token,
                to_token,
            ],
            account.address,
            deadline
        ).build_transaction(tx_data)

        return contract_txn



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/syncswap_classic_pool.json")


class SyncSwapClassicPool(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def get_pool(self, from_token, to_token):
        return self.contract.functions.getPool(
            from_token,
            to_token
        ).call()


import requests

from web3_wizzard_lib.core.utils.benchmark_utils import benchmark


class AIChat:
    def ask(self, question):
        pass


class ChatGPT:
    def __init__(self, token):
        self.token = token

    @benchmark
    def ask(self, question):
        url = "https://api.openai.com/v1/chat/completions"
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.token}"
        }
        data = {
            "model": "gpt-4o",
            "messages": [{"role": "user", "content": question}],
            "temperature": 0.7
        }

        response = requests.post(url, headers=headers, json=data)

        return response.json()['choices'][0]['message']['content']


class MockAIChat:
    def __init__(self, token):
        self.token = token

    def ask(self, question):
        return f"ANSWER {question}"


def get_ai_chat(ai_config, token):
    if ai_config == 'CHAT_GPT':
        return ChatGPT(token)
    else:
        return MockAIChat(token)



import random

from loguru import logger
from sybil_engine.config.app_config import get_cex_data, get_cex_conf
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance_utils import get_native_balance, \
    interval_to_native_balance, interval_to_erc20_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.domain.cex.binance import Binance
from sybil_engine.domain.cex.okx import OKX
from sybil_engine.module.module import Module
from sybil_engine.utils.app_account_utils import AppAccount
from sybil_engine.utils.utils import randomized_sleeping, ConfigurationException
from sybil_engine.utils.web3_utils import init_web3


class CEXWithdraw(Module):
    module_name = 'CEX_WITHDRAW'
    sleep_after_conf = True
    module_config = 'cex_withdraw'

    def execute(self, chain, withdraw_interval, min_auto_withdraw_interval, token, cex, account: AppAccount):
        self.sleep_after_conf = True
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)

        balance, min_auto_withdraw, withdraw_token = self.balance(account, chain, chain_instance,
                                                                  min_auto_withdraw_interval, token, web3)
        logger.info(f"{chain}")
        logger.info(f"Minimum balance should be {min_auto_withdraw}")

        if min_auto_withdraw.wei < balance.wei:
            self.sleep_after_conf = False

        while min_auto_withdraw.wei > balance.wei:
            logger.info(f"Actual balance: {balance}")

            amount = round(random.uniform(withdraw_interval['from'], withdraw_interval['to']), 6)
            logger.info(f"Withdraw {amount} {chain_instance['gas_token']} from {cex} to {account.address}")

            password, cex_data = get_cex_data()

            if cex == 'okx':
                cex_obj = OKX(cex_data[get_cex_conf()], password)
            elif cex == 'binance':
                cex_obj = Binance(cex_data[get_cex_conf()], password)
            else:
                raise ConfigurationException(f"{cex} is not")

            if withdraw_token == 'WETH':
                withdraw_token = 'ETH'

            cex_obj.withdrawal(account.address, chain, amount, withdraw_token)

            randomized_sleeping({'from': 60 * 2, 'to': 60 * 4})

            balance, _, _ = self.balance(account, chain, chain_instance, min_auto_withdraw_interval, token, web3)

    def balance(self, account, chain, chain_instance, min_auto_withdraw_interval, token, web3):
        if token == 'NATIVE':
            min_auto_withdraw = interval_to_native_balance(min_auto_withdraw_interval, account, chain, web3)
            balance = get_native_balance(account, web3, chain_instance)
            withdraw_token = chain_instance['gas_token']
        elif token in ['USDC', 'USDT', 'WETH', 'ETH']:
            if token == 'ETH':
                token = 'WETH'
            min_auto_withdraw = interval_to_erc20_balance(min_auto_withdraw_interval, account, token, chain, web3)
            erc20_token = Erc20Token(chain, token, web3)
            balance = erc20_token.balance(account)
            withdraw_token = token
        else:
            raise ConfigurationException(f"Unsupported token {token}, supported tokens are: NATIVE, USDC")
        return balance, min_auto_withdraw, withdraw_token

    def log(self):
        return "CEX WITHDRAW"

    def parse_params(self, module_params):
        if 'cex' not in module_params:
            module_params['cex'] = 'okx'

        return [
            module_params['chain'],
            module_params['withdraw_interval'],
            module_params['min_auto_withdraw_interval'],
            module_params['token'],
            module_params['cex']
        ]

    def sleep_after(self):
        return self.sleep_after_conf



encryption = True  # Вкл/выкл шифрования приватных ключей
password = 'test'  # Если encryption = False, то пароль не используется

proxy_mode = 'RANDOM'  # [RANDOM, STRICT]
account_create_mode = 'CSV'  # [CSV, TXT] Режим чтения аккаунтов
shell_mode = 'interactive'  # ['classic', 'interactive']
# classic - задавать модуль в module_config_default
# interactive - задавать модуль в консоли во время запуска
interactive_confirmation = True  # Если True - при запуске уточняется правильная ли конфигурация запускается

statistic_config = {
    "mode": "GOOGLE",  # CSV/GOOGLE
    "spreadsheet_id": "",  # required for GOOGLE
}

# Максимальные цены на газ Gwei для каждой сети
gas_prices = {
    'ETH_MAINNET': 50,
    'ZKSYNC': 0.26,
    'BASE': 0.5,
    'LINEA': 2,
    'ARBITRUM': 2,
    'ARBITRUM_NOVA': 2,
    'AVALANCHE': 26,
    'BSC': 5,
    'FANTOM': 750,
    'OPTIMISM': 0.5,
    'POLYGON': 150,
    'SCROLL': 2,
    'ZKFAIR': 10001,
    'COREDAO': 10,
    'MANTA': 0.3,
    'ZORA': 1,
    'POLYGON_ZK': 3,
    'X_LAYER': 10
}

# CEX configuration
cex_data = {
    'okx': '',
    'binance': ''
}
# CEX Withdraw [ZKSYNC, LINEA, BASE, ARBITRUM]
auto_withdrawal = False
withdraw_interval = {'from': 0.01, 'to': 0.015}
min_auto_withdraw_interval = {'from': 0.1, 'to': 0.2}

# Настройки телеграм.
telegram_enabled = False
telegram_api_key = ''
telegram_api_chat_id = 1
telegram_log_level = 'ERROR'  # ['INFO', 'ERROR', 'DEBUG']



from loguru import logger
from sybil_engine.contract.send import Send
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NotEnoughNativeBalance
from sybil_engine.domain.balance.balance_utils import interval_to_native_balance, interval_to_erc20_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.module.module import Module
from sybil_engine.utils.accumulator import add_accumulator_native_balance
from sybil_engine.utils.retry import retry
from sybil_engine.utils.validation_utils import validate_chain, validate_amount_interval
from sybil_engine.utils.web3_utils import init_web3


class SendToCex(Module):
    module_name = 'SEND_TO_CEX'
    module_config = 'send_to_cex_config'
    supported_chains = ['USDC', 'USDT', 'ZK', 'ZRO', 'OBT']

    def execute(self, chain, send_to_cex_amount_interval, token, account):
        cex_address = account.cex_address
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        if token == 'NATIVE':
            amount = interval_to_native_balance(send_to_cex_amount_interval, account, chain_instance['chain'], web3)

            if send_to_cex_amount_interval == 'all_balance':
                try:
                    amount = amount.minus(self.min_native_balance)
                except NotEnoughNativeBalance as e:
                    logger.info("Wallet balance is empty or almost empty, skip")
                    return
        elif token in self.supported_chains:
            amount = interval_to_erc20_balance(send_to_cex_amount_interval, account, token, chain, web3)
        else:
            raise Exception(f"Token {token} not supported. Only NATIVE and USDC/USDT are supported.")

        if amount.wei == 0 :
            logger.info("Wallet is empty")
            return

        self.send_funds(account, amount, cex_address, chain, chain_instance, token, web3)

        add_accumulator_native_balance("Total sent to cex", amount.wei)

    @retry(max_attempts=5, retry_interval={'from': 60 * 1, 'to': 60 * 2})
    def send_funds(self, account, amount, cex_address, chain, chain_instance, token, web3):
        logger.info(f"Send {amount} to {cex_address} ({chain_instance['chain']})")
        if token == 'NATIVE':
            send = Send(None, web3)
            send.send_to_wallet(account, cex_address, amount)
        elif token in self.supported_chains:
            erc20_token = Erc20Token(chain, token, web3)
            erc20_token.transfer(amount, cex_address, account)
        else:
            raise Exception(f"Token {token} not supported. Only NATIVE and USDC/USDT are supported.")

    def log(self):
        return "SEND TO CEX"

    def parse_params(self, module_params):
        validate_chain(module_params['chain'])
        validate_amount_interval(module_params['amount'])

        return module_params['chain'], module_params['amount'], module_params['token']



from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.satoshi_universe_contract import SatoshiUniverseContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class SatoshiUniverse(SubModule):
    module_name = 'SATOSHI_UNIVERSE'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['SATOSHI_UNIVERSE']
        satoshi_universe = SatoshiUniverseContract(contract_address, web3)

        satoshi_universe.mint(account, 1)

    def log(self):
        return "SATOSHI UNIVERSE NFT"



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/stargate_v2.json")


class StargateV2Contract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def send(
            self,
            account,
            value,
            send_params,
            stargate_fee,
            refund_address
    ):
        sender = account.address

        txn_params = self.build_generic_data(sender, set_contract_address=False)
        txn_params['value'] = value

        return self.contract.functions.send(
            send_params,
            stargate_fee,
            refund_address
        ).build_transaction(txn_params)

    def quote_send(self, send_params):
        return self.contract.functions.quoteSend(
            send_params,
            False
        ).call()



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/kreatorland.json")


class KreatorLandContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def mint(self, account, uri):
        txn_params = self.build_generic_data(account.address, False)

        txn_params['value'] = 660000000000000
        contract_txn = self.contract.functions.mint(
            uri,
            1,
            660000000000000
        ).build_transaction(txn_params)

        return contract_txn
